/*
 *   khexedit - Versatile hex editor
 *   Copyright (C) 1999  Espen Sand, espensa@online.no
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>
#include <values.h>

#include <qframe.h>
#include <qheader.h>
#include <qlayout.h>

#include <kapp.h>
#include <kbuttonbox.h>
#include <kcharsets.h>

#include "chartabledialog.h"
#include "dialogmargin.h"


CCharTableDialog::CCharTableDialog( QWidget *parent, const char *name )
  : QDialog( parent, name, false, WStyle_Customize|WStyle_DialogBorder )
{
  setCaption( i18n("Character Table" ) );

  QString text;
  QVBoxLayout *topLayout = new QVBoxLayout( this, OUTER_SEP, INNER_SEP );  
  if( topLayout == 0 ) { return; }

  mCharacterList = new QListView( this, "characterList" );
  mCharacterList->setFont( kapp->fixedFont ); 
  mCharacterList->addColumn( i18n("Decimal") );
  mCharacterList->addColumn( i18n("Hexadecimal") );
  mCharacterList->addColumn( i18n("Octal") );
  mCharacterList->addColumn( i18n("Binary") );
  mCharacterList->addColumn( i18n("Text") );
  mCharacterList->setAllColumnsShowFocus( true );
  mCharacterList->setFrameStyle( QFrame::WinPanel + QFrame::Sunken );
  mCharacterList->setSorting( -1 );
  //
  // I am setting the min. size below
  //
  connect( mCharacterList, SIGNAL(doubleClicked(QListViewItem *) ),
	   SLOT(startAssign(QListViewItem *)));
  topLayout->addWidget( mCharacterList, 10 );
  
  QHBoxLayout *hbox = new QHBoxLayout( INNER_SEP );
  if( hbox == 0 ) { return; }
  topLayout->addLayout( hbox );
  
  text = i18n("Insert this number of characters:");
  QLabel *label = new QLabel( text, this );
  label->setFixedSize( label->sizeHint() );
  hbox->addWidget( label );
  
  mInputCountSpin = new QSpinBox( this, "spin" );
  mInputCountSpin->setMinimumWidth( fontMetrics().width("M") * 10 );
  mInputCountSpin->setFixedHeight( mInputCountSpin->sizeHint().height() );
  mInputCountSpin->setRange( 1, MAXINT );
  mInputCountSpin->setValue( 1 );
  hbox->addWidget( mInputCountSpin );
  
  hbox->addStretch ( 10 );  

  QFrame *hline = new QFrame( this );
  hline->setFrameStyle( QFrame::Sunken | QFrame::HLine );
  hline->setFixedHeight( hline->sizeHint().height() );
  topLayout->addWidget( hline );


  topLayout->addSpacing( ACTION_SPACE );

  KButtonBox *buttonBox = new KButtonBox( this, KButtonBox::HORIZONTAL, 0, 
					  INNER_SEP );

  QPushButton *helpButton = buttonBox->addButton( i18n("&Help"), false );
  buttonBox->addStretch();
  QPushButton *startButton = buttonBox->addButton( i18n("&Insert"), false );
  QPushButton *cancelButton = buttonBox->addButton( i18n("&Cancel"), false );
  buttonBox->layout();
  startButton->setDefault( true );
  connect( cancelButton, SIGNAL(clicked()),this,SLOT(reject()) );
  connect( helpButton, SIGNAL(clicked()),this,SLOT(help()) );
  connect( startButton, SIGNAL(clicked()),this,SLOT(startAssign()) );
  topLayout->addWidget( buttonBox );

  createListData();

  topLayout->activate();
  setGeometry( x(), y(), minimumSize().width(), minimumSize().height() );

  setColumnWidth();

  mCharacterList->setMinimumSize(
    mCharacterList->sizeHint().width() +
    mCharacterList->verticalScrollBar()->sizeHint().width() + 
    mCharacterList->lineWidth() * 2,
    mCharacterList->fontMetrics().lineSpacing() * 20 );

  mCharacterList->viewport()->update();

  setColumnWidth();

}



CCharTableDialog::~CCharTableDialog( void )
{
}


const char *printBinary( uint val )
{
  static char buf[9];
  for( int i = 0; i < 8; i++ )
  {
    buf[7-i] = (val&(1<<i)) ? '1' : '0';
  }
  buf[8] = 0;
  return( buf );
}


void CCharTableDialog::createListData( void )
{
  QString d, h, o, b, c;
  QListViewItem *item = 0;

  char buf[10];
  memset( buf, 0, sizeof( buf ) );
  
  QFontInfo info( font() );
  KCharset charset( info.charSet() );

  for( uint i=0; i<256; i++ )
  { 
    d.sprintf("%03d", i );
    h.sprintf("0x%02x", i );
    o.sprintf("%03o", i );
    b.sprintf("%s", printBinary(i) );

    if( charset.printable(i) == true )
    {
      c.sprintf("%c", i );
    }
    else
    {
      c.sprintf("%c", '.' );
    }

    item = new QListViewItem( mCharacterList, item, d, h, o, b, c );
    if( i == 0 )
    {
      mCharacterList->setSelected( item, true );
    } 

  }
}


void CCharTableDialog::help( void )
{

}

void CCharTableDialog::startAssign( void )
{
 QListViewItem *item = mCharacterList->currentItem();
 if( item != 0 )
 {
   startAssign( item );
 }

}

void CCharTableDialog::startAssign( QListViewItem *item )
{
  const char *text = item->text(0);
  if( text == 0 )
  {
    return;
  }

  QByteArray buf;
  if( buf.fill( atoi(text), mInputCountSpin->value() ) == false )
  {
    return;
  }
  emit assign( buf );
}


void CCharTableDialog::setColumnWidth( void )
{
  const QFontMetrics &fm = mCharacterList->fontMetrics();
  int w0, w1, w2, w3;

  w0 = -fm.minLeftBearing() - fm.minRightBearing() + 8 + fm.maxWidth();
  w3 = 0;

  w1  = fm.width( mCharacterList->header()->label(0) ) + w0;
  w2  = fm.width('0')*5;
  w3 += w1 > w2 ? w1 : w2;
  mCharacterList->setColumnWidth( 0, w1 > w2 ? w1 : w2 );
  
  w1  = fm.boundingRect( mCharacterList->header()->label(1) ).width() + w0;
  w2  = fm.width('0')*6;
  w3 += w1 > w2 ? w1 : w2;
  mCharacterList->setColumnWidth( 1, w1 > w2 ? w1 : w2 );

  w1  = fm.boundingRect( mCharacterList->header()->label(2) ).width() + w0;
  w2  = fm.width('0')*5;
  w3 += w1 > w2 ? w1 : w2;
  mCharacterList->setColumnWidth( 2, w1 > w2 ? w1 : w2 );

  w1  = fm.boundingRect( mCharacterList->header()->label(3) ).width() + w0;
  w2  = fm.width('0')*10;
  w3 += w1 > w2 ? w1 : w2;
  mCharacterList->setColumnWidth( 3, w1 > w2 ? w1 : w2 );

  //
  // Bad trick: The mCharacterList->viewport()->width() 
  // becomes far too large at the beginning so a horizontal scrollbar
  // becomes visible.
  // This file is a backport from Qt-2 (where it works) so I keep the
  // hack for now.
  // Normally I want the last label to stretch to the right border
  //
  w1 = fm.boundingRect( mCharacterList->header()->label(4) ).width() + w0;
  w2 = fm.width('0')*3;
  mCharacterList->setColumnWidth( 4, w1 > w2 ? w1 : w2 );

  /*
  int w4 = mCharacterList->viewport()->width() - w3;
  w1 = fm.boundingRect( mCharacterList->header()->label(4) ).width() + w0;
  w2 = fm.width('0')*3;
  w1 = w1 > w2 ? w1 : w2;
  mCharacterList->setColumnWidth( 4, w1 > w4 ? w1 : w4 );
  */

}




void CCharTableDialog::resizeEvent( QResizeEvent */*e*/ )
{
  setColumnWidth();
}




