/*
 *   khexedit - Versatile hex editor
 *   Copyright (C) 1999  Espen Sand, espensa@online.no
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <values.h>

#include <qbttngrp.h>
#include <qfont.h>
#include <qlayout.h>
#include <qlabel.h>

#include <kapp.h>
#include <kcolordlg.h>

#include "dialogmargin.h"
#include "optiondialog.h"


COptionDialog::COptionDialog( const char *title, QWidget *parent, 
			      char *name, bool modal )
  : CPageDialog( CPageDialog::TREE,  CPageDialog::HELP | CPageDialog::DEFAULT |
		 CPageDialog::OK | CPageDialog::APPLY | CPageDialog::CANCEL,
		 CPageDialog::OK, title, parent, name, modal )
{
  //QString longTitle;
  const char *longTitle;

  longTitle = i18n("Data layout in editor.");
  mFrame[ page_layout ] = addPage( -1, i18n("Layout"), longTitle );
  longTitle = i18n("Cursor behavior. Only valid for editor.");
  mFrame[ page_cursor ] = addPage( -1, i18n("Cursor"), longTitle );
  longTitle = i18n("Editor colors. System selection color is always used.");
  mFrame[ page_color ]  = addPage( -1, i18n("Colors"), longTitle );
  longTitle = i18n("Font selection. Editor can only use a fixed font.");
  mFrame[ page_font ]   = addPage( -1, i18n("Font"), longTitle );
  longTitle = i18n("File management");
  mFrame[ page_file ] = addPage( -1, i18n("Files"), longTitle );
  longTitle = i18n("Various properties.");
  mFrame[ page_misc ] = addPage( -1, i18n("Miscellaneous"), longTitle );

  setupLayoutPage();
  setupCursorPage();
  setupColorPage();
  setupFontPage();
  setupFilePage();
  setupMiscPage();

  connect( this, SIGNAL(defaultClicked()), SLOT(slotDefault()));
  connect( this, SIGNAL(applyClicked()), SLOT(slotApply()));
  
  adjustPageSize();
  adjustSize();
  showPage( page_layout );
}

COptionDialog::~COptionDialog( void )
{
}




void COptionDialog::setupLayoutPage( void )
{
  const char *text;
  QFrame *page = mFrame[ page_layout ];

  QVBoxLayout *topLayout = new QVBoxLayout( page, OUTER_SEP, INNER_SEP );
  if( topLayout == 0 ) { return; }

  QFrame *frame = new QFrame( page );
  if( frame == 0 ) { return; }
  topLayout->addWidget( frame, 0, AlignLeft );
  QGridLayout *gbox = new QGridLayout( frame, 14, 2, 0, INNER_SEP );
  if( gbox == 0 ) { return; }
  gbox->setColStretch( 1, 10 );  

  mLayout.formatCombo  = new QComboBox( false, frame );
  mLayout.formatCombo->setFixedHeight( 
    mLayout.formatCombo->sizeHint().height() );
  mLayout.formatCombo->setMinimumWidth( 
    mLayout.formatCombo->sizeHint().width() );
  const char *items[] = 
  { 
    i18n("Hexadecimal mode"),
    i18n("Octal mode"),
    i18n("Binary mode"),
    i18n("Text only mode"),
    0 
  };
  mLayout.formatCombo->insertStrList( items );
  connect( mLayout.formatCombo, SIGNAL(activated(int)), 
	   SLOT(slotModeSelectorChanged(int)) );
  gbox->addWidget( mLayout.formatCombo, 0, 0 );

  mLayout.lineSizeSpin = new QSpinBox( frame );
  mLayout.lineSizeSpin->setFixedHeight( 
    mLayout.lineSizeSpin->sizeHint().height() );
  mLayout.lineSizeSpin->setMinimumWidth( fontMetrics().width("M") * 10 );
  mLayout.lineSizeSpin->setRange( 1, 10000 );
  connect( mLayout.lineSizeSpin, SIGNAL(valueChanged(int)), 
	   SLOT(slotLineSizeChanged(int) ) );
  gbox->addWidget( mLayout.lineSizeSpin, 1, 1 );

  mLayout.columnSizeSpin = new QSpinBox( frame );
  mLayout.columnSizeSpin->setFixedHeight( 
    mLayout.columnSizeSpin->sizeHint().height() );
  mLayout.columnSizeSpin->setMinimumWidth( fontMetrics().width("M") * 10 );
  mLayout.columnSizeSpin->setRange( 1, 10000 );
  connect( mLayout.columnSizeSpin, SIGNAL(valueChanged(int)), 
	   SLOT(slotColumnSizeChanged(int) ) );
  gbox->addWidget( mLayout.columnSizeSpin, 2, 1 );

  text = i18n("Default l&ine size [bytes]");
  QLabel *label = new QLabel( mLayout.lineSizeSpin, text, frame );
  label->setMinimumWidth( label->sizeHint().width() );
  gbox->addWidget( label, 1, 0 );

  text = i18n("Colu&mn size [bytes]");
  label = new QLabel( mLayout.columnSizeSpin, text, frame );
  label->setMinimumWidth( label->sizeHint().width() );
  gbox->addWidget( label, 2, 0 );

  QFrame *hline = new QFrame( frame );
  hline->setFrameStyle( QFrame::Sunken | QFrame::HLine );
  hline->setFixedHeight( hline->sizeHint().height() );
  gbox->addMultiCellWidget( hline, 3, 3, 0, 1, AlignLeft );

  text = i18n("Line size is &fixed (use scrollbar when required)"); 
  mLayout.lockLineCheck = new QCheckBox( text, frame );
  mLayout.lockLineCheck->setFixedSize( mLayout.lockLineCheck->sizeHint() );
  gbox->addMultiCellWidget( mLayout.lockLineCheck, 4, 4, 0, 1, AlignLeft );

  text = i18n("Loc&k column at end of line (when column size > 1)"); 
  mLayout.lockColumnCheck = new QCheckBox( text, frame );
  mLayout.lockColumnCheck->setFixedSize( mLayout.lockColumnCheck->sizeHint() );
  gbox->addMultiCellWidget( mLayout.lockColumnCheck, 5, 5, 0, 1, AlignLeft );

  hline = new QFrame( frame );
  hline->setFrameStyle( QFrame::Sunken | QFrame::HLine );
  hline->setFixedHeight( hline->sizeHint().height() );
  gbox->addMultiCellWidget( hline, 6, 6, 0, 1, AlignLeft );

  const char *gridMode[] = 
  { 
    i18n("None"),
    i18n("Vertical only"),
    i18n("Horizontal only"),
    i18n("Both directions"),
    0 
  };
  mLayout.gridCombo = new QComboBox( false, frame );
  mLayout.gridCombo->setFixedHeight( 
    mLayout.gridCombo->sizeHint().height() );
  mLayout.gridCombo->setMinimumWidth( 
    mLayout.gridCombo->sizeHint().width() );
  mLayout.gridCombo->insertStrList( gridMode );

  text = i18n("&Gridlines between text"); 
  label = new QLabel( mLayout.gridCombo, text, frame );
  label->setMinimumWidth( label->sizeHint().width() );

  gbox->addWidget( label, 7, 0 );
  gbox->addWidget( mLayout.gridCombo, 7, 1 );

  mLayout.leftSepWidthSpin = new QSpinBox( frame );
  mLayout.leftSepWidthSpin->setFixedHeight( 
    mLayout.leftSepWidthSpin->sizeHint().height() );
  mLayout.leftSepWidthSpin->setMinimumWidth( fontMetrics().width("M") * 10 );
  mLayout.leftSepWidthSpin->setRange( 0, 20 );
  gbox->addWidget( mLayout.leftSepWidthSpin, 8, 1 );

  mLayout.rightSepWidthSpin = new QSpinBox( frame );
  mLayout.rightSepWidthSpin->setFixedHeight( 
    mLayout.rightSepWidthSpin->sizeHint().height() );
  mLayout.rightSepWidthSpin->setMinimumWidth( fontMetrics().width("M") * 10 );
  mLayout.rightSepWidthSpin->setRange( 0, 20 );
  gbox->addWidget( mLayout.rightSepWidthSpin, 9, 1 );

  text = i18n("&Left separator width [pixels]");
  mLayout.leftSepLabel = new QLabel( mLayout.leftSepWidthSpin, text, frame );
  mLayout.leftSepLabel->setMinimumWidth( 
    mLayout.leftSepLabel->sizeHint().width() );
  gbox->addWidget( mLayout.leftSepLabel, 8, 0 );

  text = i18n("&Right separator width [pixels]");
  mLayout.rightSepLabel = new QLabel( mLayout.rightSepWidthSpin, text, frame );
  mLayout.rightSepLabel->setMinimumWidth( 
    mLayout.rightSepLabel ->sizeHint().width() );
  gbox->addWidget( mLayout.rightSepLabel, 9, 0 );

  mLayout.separatorSpin = new QSpinBox( frame );
  mLayout.separatorSpin->setFixedHeight( 
    mLayout.separatorSpin->sizeHint().height() );
  mLayout.separatorSpin->setMinimumWidth( fontMetrics().width("M") * 10 );
  mLayout.separatorSpin->setRange( 0, 20 );
  gbox->addWidget( mLayout.separatorSpin, 10, 1 );

  mLayout.edgeSpin = new QSpinBox( frame );
  mLayout.edgeSpin->setFixedHeight( mLayout.edgeSpin->sizeHint().height() );
  mLayout.edgeSpin->setMinimumWidth( fontMetrics().width("M") * 10 );
  mLayout.edgeSpin->setRange( 0, 20 );
  gbox->addWidget( mLayout.edgeSpin, 11, 1 );

  text = i18n("&Separator margin width [pixels]");
  label = new QLabel( mLayout.separatorSpin, text, frame );
  label->setMinimumWidth( label->sizeHint().width() );
  gbox->addWidget( label, 10, 0 );

  text = i18n("&Edge margin width [pixels]");
  label = new QLabel( mLayout.edgeSpin, text, frame );
  label->setMinimumWidth( label->sizeHint().width() );
  gbox->addWidget( label, 11, 0 );
 
  text = i18n("Column separation is e&qual to one character");
  mLayout.columnCheck = new QCheckBox( text, frame );
  mLayout.columnCheck->setFixedSize( mLayout.columnCheck->sizeHint() );
  mLayout.columnCheck->setMinimumSize( mLayout.columnCheck->sizeHint() );
  gbox->addMultiCellWidget( mLayout.columnCheck, 12, 12, 0, 1, AlignLeft );
  connect( mLayout.columnCheck, SIGNAL(toggled(bool)),
	   SLOT( slotColumnSepCheck(bool)));
 
  mLayout.columnSepSpin = new QSpinBox( frame );
  mLayout.columnSepSpin->setFixedHeight( 
    mLayout.columnSepSpin->sizeHint().height() );
  mLayout.columnSepSpin->setMinimumWidth( fontMetrics().width("M") * 10 );
  mLayout.columnSepSpin->setRange( 1, 100 );
 
  text = i18n("Column separa&tion [pixels]");
  mLayout.columnSepLabel = new QLabel( mLayout.columnSepSpin, text, frame );
  mLayout.columnSepLabel->setMinimumWidth( 
    mLayout.columnSepLabel->sizeHint().width() );

  gbox->addWidget( mLayout.columnSepLabel, 13, 0 );
  gbox->addWidget( mLayout.columnSepSpin, 13, 1 );
  gbox->activate();


  QWidget *labelList[3] = 
  { 
    mLayout.lockLineCheck, mLayout.lockColumnCheck, mLayout.columnCheck
  };
  int maxWidth = 0;
  for( int i=0; i<3; i++)
  {
    int w = labelList[i]->sizeHint().width();
    if( w > maxWidth ) { maxWidth = w; }
  }
  frame->setMinimumWidth( maxWidth );


  topLayout->addStretch(10);
  topLayout->activate();
}


void COptionDialog::setupCursorPage( void )
{
  const char *text;
  QFrame *page = mFrame[ page_cursor ];

  QVBoxLayout *topLayout = new QVBoxLayout( page, OUTER_SEP, INNER_SEP );
  if( topLayout == 0 ) { return; }


  QGroupBox *group = new QGroupBox( page );
  if( group == 0 ) { return; }
  topLayout->addWidget( group );
  group->setTitle( i18n("Blinking") );
  group->setFrameStyle( QFrame::Box|QFrame::Sunken );
  QVBoxLayout *vbox = new QVBoxLayout( group, 5 );
  vbox->addSpacing( group->fontMetrics().height() );
  text = i18n("Do not b&link");
  mCursor.blinkCheck = new QCheckBox( text, group );
  mCursor.blinkCheck->setFixedHeight(mCursor.blinkCheck->sizeHint().height());
  connect( mCursor.blinkCheck, SIGNAL(toggled(bool)),
	   SLOT( slotBlinkIntervalCheck(bool)));
  vbox->addWidget( mCursor.blinkCheck );
  
  QHBoxLayout *hbox = new QHBoxLayout();
  if( hbox == 0 ) { return; }
  vbox->addLayout( hbox );
  mCursor.blinkSpin = new QSpinBox( group );
  mCursor.blinkSpin->setFixedHeight( mCursor.blinkSpin->sizeHint().height() );
  mCursor.blinkSpin->setMinimumWidth( fontMetrics().width("M") * 10 );
  mCursor.blinkSpin->setRange( 100, 1000 );
  mCursor.blinkSpin->setSteps( 100, 100 );
  mCursor.blinkSpin->setValue( 500 );
  text = i18n("&Blink interval [ms]" );
  mCursor.blinkLabel = new QLabel( mCursor.blinkSpin, text, group );
  mCursor.blinkLabel->setMinimumWidth( mCursor.blinkLabel->sizeHint().width());
  hbox->addWidget( mCursor.blinkLabel );
  hbox->addWidget( mCursor.blinkSpin, 10 );
  vbox->activate();
  
  group = new QGroupBox( page );
  if( group == 0 ) { return; }
  topLayout->addWidget( group );
  group->setTitle( i18n("Shape") );
  group->setFrameStyle( QFrame::Box|QFrame::Sunken );
  vbox = new QVBoxLayout( group, 5 );
  vbox->addSpacing( group->fontMetrics().height() );
  text = i18n("Always &use block (rectangular) cursor");
  mCursor.blockCheck = new QCheckBox( text, group );
  mCursor.blockCheck->setFixedHeight( mCursor.blockCheck->sizeHint().height());
  connect( mCursor.blockCheck, SIGNAL(toggled(bool)),
	   SLOT( slotBlockCursorCheck(bool)));
  vbox->addWidget( mCursor.blockCheck );
  text = i18n("Use &thick cursor in insert mode");
  mCursor.thickCheck = new QCheckBox( text, group );
  mCursor.thickCheck->setFixedHeight( mCursor.thickCheck->sizeHint().height());
  vbox->addWidget( mCursor.thickCheck );
  vbox->activate();

  QButtonGroup *buttonGroup = new QButtonGroup( page );
  if( buttonGroup == 0 ) { return; }
  buttonGroup->setTitle( i18n("Cursor behavior when editor looses focus") );
  topLayout->addWidget( buttonGroup, 0, AlignLeft );
  vbox = new QVBoxLayout( buttonGroup, 5 );
  vbox->addSpacing( buttonGroup->fontMetrics().height() );
  mCursor.stopRadio = new QRadioButton( "radio1", buttonGroup );
  text = i18n("&Stop blinking (if blinking is enabled)");
  mCursor.stopRadio->setText( text );
  mCursor.stopRadio->setMinimumSize( mCursor.stopRadio->sizeHint() );
  vbox->addWidget( mCursor.stopRadio );
  mCursor.hideRadio = new QRadioButton( "radio2", buttonGroup );
  mCursor.hideRadio->setText(i18n("H&ide") );
  mCursor.hideRadio->setMinimumSize( mCursor.hideRadio->sizeHint() );
  vbox->addWidget( mCursor.hideRadio );
  mCursor.nothingRadio = new QRadioButton( "radio3", buttonGroup );
  mCursor.nothingRadio->setText(i18n("Do &nothing") );
  mCursor.nothingRadio->setMinimumSize( mCursor.nothingRadio->sizeHint() );
  vbox->addWidget( mCursor.nothingRadio );
  vbox->activate();

  topLayout->addStretch(10);
  topLayout->activate();
}

void COptionDialog::setupColorPage( void )
{
  const char *text;
  QFrame *page = mFrame[ page_color ];

  QVBoxLayout *topLayout = new QVBoxLayout( page, OUTER_SEP, INNER_SEP );
  if( topLayout == 0 ) { return; }

  text = i18n("&Use system colors (as chosen in Control Center)");
  mColor.checkSystem = new QCheckBox( text, page );
  mColor.checkSystem->setMinimumSize( mColor.checkSystem->sizeHint() );
  connect( mColor.checkSystem, SIGNAL(toggled(bool)),
	   SLOT( slotColorSystem(bool)));
  topLayout->addWidget( mColor.checkSystem, 0, AlignLeft );

  QFrame *hline = new QFrame( page );
  hline->setFrameStyle( QFrame::Sunken | QFrame::HLine );
  hline->setFixedHeight( hline->sizeHint().height() );
  topLayout->addWidget( hline );

  const char *itemList[] = 
  { 
    i18n("First, third ... line background"),
    i18n("Second, fourth ... line background"),
    i18n("Offset background"),
    i18n("Inactive background"),
    i18n("Even column text"),
    i18n("Odd column text"),
    i18n("Non printable text"),
    i18n("Offset text"),
    i18n("Secondary text"),
    i18n("Marked background"),
    i18n("Marked text"),
    i18n("Cursor background"),
    i18n("Cursor text (block shape)"),
    i18n("Separator"),
    i18n("Grid lines"),
    0
  };
  mColor.colorList = new CColorListBox( page );
  topLayout->addWidget( mColor.colorList, 10 );
  for( uint i=0; i<15; i++ )
  {
    CColorListItem *listItem = new CColorListItem( itemList[i] );
    mColor.colorList->insertItem( listItem );
  }
  mColor.colorList->setCurrentItem(0);
  topLayout->activate();
}


void COptionDialog::setupFontPage( void )
{
  const char *text;
  QFrame *page = mFrame[ page_font ];

  QVBoxLayout *topLayout = new QVBoxLayout( page, OUTER_SEP, INNER_SEP );
  if( topLayout == 0 ) { return; }

  text = i18n("&Use system font (as chosen in Control Center)");
  mFont.checkSystem = new QCheckBox( text, page );
  mFont.checkSystem->setFixedHeight( mFont.checkSystem->sizeHint().height());
  connect( mFont.checkSystem, SIGNAL(toggled(bool)),
	   SLOT( slotFontSystem(bool)));
  topLayout->addWidget( mFont.checkSystem );

  QFrame *frame = new QFrame( page );
  if( frame == 0 ) { return; }
  frame->setFrameStyle( QFrame::Box|QFrame::Sunken );
  topLayout->addWidget( frame );

  QBoxLayout *hbox = new QHBoxLayout( frame, 2 );
  if( hbox == 0 ) { return; }
  mFont.chooser = new KFontChooser( frame, "font" );
  if( mFont.chooser == 0 ) { return; }
  hbox->addWidget( mFont.chooser );

  hbox = new QHBoxLayout();
  if( hbox == 0 ) { return; }
  topLayout->addLayout( hbox );
  
  mFont.nonPrintInput = new QLineEdit( page );
  if( mFont.nonPrintInput == 0 ) { return; }
  mFont.nonPrintInput->setFixedHeight(
    mFont.nonPrintInput->sizeHint().height() );
  mFont.nonPrintInput->setMaxLength( 1 );
 
  text = i18n("&Map non printable characters to:");
  QLabel *nonPrintLabel = new QLabel( mFont.nonPrintInput, text, page );
  if( nonPrintLabel == 0 ) { return; }
  nonPrintLabel->setFixedHeight( mFont.nonPrintInput->sizeHint().height() );
  nonPrintLabel->setFixedWidth( nonPrintLabel->sizeHint().width() );
  
  hbox->addWidget( nonPrintLabel, 0, AlignLeft );
  hbox->addSpacing( INNER_SEP );
  hbox->addWidget( mFont.nonPrintInput, 10 );


  mFont.chooser->setFont( QFont( "courier", 14, QFont::Bold, FALSE ), true );

  topLayout->addStretch(10);
  topLayout->activate();
}



void COptionDialog::setupFilePage( void )
{
  const char *text;
  QFrame *page = mFrame[ page_file ];

  QVBoxLayout *topLayout = new QVBoxLayout( page, OUTER_SEP, INNER_SEP );

  QGridLayout *gbox = new QGridLayout( 2, 2, INNER_SEP );
  if( gbox == 0 ) { return; }
  topLayout->addLayout( gbox );

  mFile.openCombo = new QComboBox( false, page );
  const char *items[] = 
  { 
    i18n( "None" ),
    i18n( "Most recent file" ),
    i18n( "All recent files" ),
    0,
  };
  mFile.openCombo->insertStrList( items );
  mFile.openCombo->setFixedWidth( mFile.openCombo->sizeHint().width()+10 );
  mFile.openCombo->setFixedHeight( mFile.openCombo->sizeHint().height() );


  text = i18n("Open doc&uments on startup");
  QLabel *label = new QLabel( mFile.openCombo, text, page );
  label->setFixedHeight( label->sizeHint().height() );
  label->setMinimumWidth( label->sizeHint().width() );

  gbox->addWidget( label, 0, 0 );
  gbox->addWidget( mFile.openCombo, 1, 0 );

  text = i18n("&Jump to previous cursor position on startup");
  mFile.gotoOffsetCheck = new QCheckBox( text, page );
  mFile.gotoOffsetCheck->setFixedSize( mFile.gotoOffsetCheck->sizeHint() );
  topLayout->addWidget( mFile.gotoOffsetCheck, 0, AlignLeft );

  QFrame *hline = new QFrame( page );
  hline->setFrameStyle( QFrame::Sunken | QFrame::HLine );
  hline->setFixedHeight( hline->sizeHint().height() );
  topLayout->addWidget( hline );

  text = i18n("Open document with &write proctection enabled");
  mFile.writeProtectCheck = new QCheckBox( text, page );
  mFile.writeProtectCheck->setFixedSize( mFile.writeProtectCheck->sizeHint() );
  topLayout->addWidget( mFile.writeProtectCheck, 0, AlignLeft );

  text = i18n("&Keep cursor position after reloading document");
  mFile.reloadOffsetCheck = new QCheckBox( text, page );
  mFile.reloadOffsetCheck->setFixedSize( mFile.reloadOffsetCheck->sizeHint() );
  topLayout->addWidget( mFile.reloadOffsetCheck, 0, AlignLeft );

  text = i18n("&Make a backup when saving document");
  mFile.backupCheck = new QCheckBox( text, page );
  mFile.backupCheck->setFixedSize( mFile.backupCheck->sizeHint() );
  topLayout->addWidget( mFile.backupCheck, 0, AlignLeft );

  hline = new QFrame( page );
  hline->setFrameStyle( QFrame::Sunken | QFrame::HLine );
  hline->setFixedHeight( hline->sizeHint().height() );
  topLayout->addWidget( hline );

  text = i18n("Don't &save \"Recent\" document list on exit");
  mFile.discardRecentCheck = new QCheckBox( text, page );
  mFile.discardRecentCheck->setFixedSize(mFile.discardRecentCheck->sizeHint());
  topLayout->addWidget( mFile.discardRecentCheck, 0, AlignLeft );
 
  text = i18n("Clear \"Recent\" document list &now");
  QPushButton *discardRecentButton = new QPushButton( page );
  discardRecentButton->setText( text );
  discardRecentButton->setFixedSize( discardRecentButton->sizeHint() );
  topLayout->addWidget( discardRecentButton, 0, AlignLeft );
  connect( discardRecentButton, SIGNAL(clicked()),
	   SIGNAL(removeRecentFiles()) );

  topLayout->addStretch(10);
  topLayout->activate();
}



void COptionDialog::setupMiscPage( void )
{
  const char *text;
  QFrame *page = mFrame[ page_misc ];

  QVBoxLayout *topLayout = new QVBoxLayout( page, OUTER_SEP, INNER_SEP );

  
  QFrame *f1 = new QFrame( page );
  if( f1 == 0 ) { return; }
  topLayout->addWidget( f1 );

  QGridLayout *gbox = new QGridLayout( f1, 2, 2, 0, INNER_SEP );
  if( gbox == 0 ) { return; }

  mMisc.undoSpin = new QSpinBox( f1 );
  mMisc.undoSpin->setFixedHeight( mMisc.undoSpin->sizeHint().height() );
  mMisc.undoSpin->setRange( 10, 10000 );
  mMisc.undoSpin->setSteps( 5, 5 );
  mMisc.undoSpin->setValue( 50 );
  QLabel *label = new QLabel( mMisc.undoSpin, i18n("&Undo history"), f1 );
  label->setFixedHeight( label->sizeHint().height() );
  label->setMinimumWidth( label->sizeHint().width() );

  gbox->addWidget( label, 0, 0 );
  gbox->addWidget( mMisc.undoSpin, 1, 0 );

  QGroupBox *group = new QGroupBox( page );
  if( group == 0 ) { return; }
  group->setTitle( i18n("Sounds") );
  topLayout->addWidget( group );
  QVBoxLayout *vbox = new QVBoxLayout( group, 5 );
  vbox->addSpacing( group->fontMetrics().height() );
  text = i18n("Make sound on data &input (eg. typing) failure");
  mMisc.inputCheck = new QCheckBox( text, group );
  mMisc.inputCheck->setFixedSize( mMisc.inputCheck->sizeHint() );
  vbox->addWidget( mMisc.inputCheck, 0, AlignLeft );
  text = i18n("Make sound on &fatal failure");
  mMisc.fatalCheck = new QCheckBox( text, group );
  mMisc.fatalCheck->setFixedSize( mMisc.fatalCheck->sizeHint() );
  vbox->addWidget( mMisc.fatalCheck, 0, AlignLeft );
  vbox->activate();

  text = i18n("Auto&matic copy to clipboard when selection is ready");
  mMisc.autoCheck = new QCheckBox( text, page );
  mMisc.autoCheck->setFixedSize( mMisc.autoCheck->sizeHint() );
  topLayout->addWidget( mMisc.autoCheck, 0, AlignLeft );

  text = i18n("&Editor starts in \"insert\" mode" );
  mMisc.insertCheck = new QCheckBox( text, page );
  mMisc.insertCheck->setFixedHeight( mMisc.insertCheck->sizeHint().height() );
  topLayout->addWidget( mMisc.insertCheck, 0, AlignLeft );

  text = i18n("Confirm &wrapping (to beginning or end) during search");
  mMisc.confirmWrapCheck = new QCheckBox( text, page );
  mMisc.confirmWrapCheck->setMinimumSize( mMisc.confirmWrapCheck->sizeHint() );
  topLayout->addWidget( mMisc.confirmWrapCheck, 0, AlignLeft );

  text = i18n("Cursor jumps to &nearest byte when moved");
  mMisc.cursorJumpCheck = new QCheckBox( text, page );
  mMisc.cursorJumpCheck->setFixedSize( mMisc.cursorJumpCheck->sizeHint() );
  topLayout->addWidget( mMisc.cursorJumpCheck, 0, AlignLeft );

  QFrame *hline = new QFrame( page );
  hline->setFrameStyle( QFrame::Sunken | QFrame::HLine );
  hline->setFixedHeight( hline->sizeHint().height() );
  topLayout->addWidget( hline );

  text = i18n("Confirm when number of printed pages will e&xceed limit");
  mMisc.thresholdCheck = new QCheckBox( text, page );
  mMisc.thresholdCheck->setFixedSize( mMisc.thresholdCheck->sizeHint() );
  connect( mMisc.thresholdCheck, SIGNAL(clicked()),
	   SLOT( slotThresholdConfirm()));
  topLayout->addWidget( mMisc.thresholdCheck, 0, AlignLeft );

  QHBoxLayout *hbox = new QHBoxLayout(INNER_SEP);
  if( hbox == 0 ) { return; }
  topLayout->addLayout( hbox );

  mMisc.thresholdSpin = new QSpinBox( page );
  mMisc.thresholdSpin->setFixedHeight( 
    mMisc.thresholdSpin->sizeHint().height() );
  mMisc.thresholdSpin->setMinimumWidth( fontMetrics().width("M") * 10 );
  mMisc.thresholdSpin->setRange( 5, MAXINT );
  mMisc.thresholdSpin->setSteps( 5, 5 );
  mMisc.thresholdSpin->setValue( 5 );

  text = i18n("&Threshold [pages]" );
  mMisc.thresholdLabel = new QLabel( mMisc.thresholdSpin, text, page );
  mMisc.thresholdLabel->setFixedWidth(
    mMisc.thresholdLabel->sizeHint().width() );
  
  hbox->addWidget( mMisc.thresholdLabel, 0, AlignLeft );
  hbox->addWidget( mMisc.thresholdSpin, 0, AlignLeft );
  hbox->addStretch(10);

  topLayout->addStretch(10);
  topLayout->activate();
}



void COptionDialog::slotModeSelectorChanged( int index )
{
  int lineSize = mDisplayState.line.getLineSize( index );
  int colSize  = mDisplayState.line.getColumnSize( index );
  mLayout.lineSizeSpin->setValue( lineSize );
  mLayout.columnSizeSpin->setValue( colSize );
}

void COptionDialog::slotLineSizeChanged( int value )
{
  int index = mLayout.formatCombo->currentItem();
  mDisplayState.line.setLineSize( index, value );
  if( index == 3 ) { mLayout.columnSizeSpin->setValue( value ); }
}

void COptionDialog::slotColumnSizeChanged( int value )
{
  int index = mLayout.formatCombo->currentItem();
  mDisplayState.line.setColumnSize( index, value );
  if( index == 3 ) { mLayout.lineSizeSpin->setValue( value ); }
}

void COptionDialog::slotColumnSepCheck( bool state )
{
  state = state == true ? false: true;
  mLayout.columnSepLabel->setEnabled( state );
  mLayout.columnSepSpin->setEnabled( state );
}


void COptionDialog::slotBlinkIntervalCheck( bool state )
{
  state = state == true ? false : true;
  mCursor.blinkSpin->setEnabled( state );
  mCursor.blinkLabel->setEnabled( state );
}

void COptionDialog::slotBlockCursorCheck( bool state )
{
  state = state == true ? false : true;
  mCursor.thickCheck->setEnabled( state );
}

void COptionDialog::slotFontSystem( bool state )
{
  mFont.chooser->setEnabled( state == true ? false : true );
}

void COptionDialog::slotColorSystem( bool state )
{
  state = state == true ? false : true;
  mColor.colorList->setEnabled( state );
}


void COptionDialog::slotThresholdConfirm( void )
{
  bool state = mMisc.thresholdCheck->isChecked();
  mMisc.thresholdLabel->setEnabled( state );
  mMisc.thresholdSpin->setEnabled( state );
}



void COptionDialog::slotHelp( void )
{
  kapp->invokeHTMLHelp( QString("khexedit/khexedit.html"), QString() );
}


void COptionDialog::slotDefault( void )
{
  //
  // We use the constructor settings as default values
  //
  switch( activePage() )
  {
    case page_layout:
    {
      SDisplayLayout layout; 
      SDisplayLine line; 
      setLayout( layout, line );
    }
    break;

    case page_cursor:
    {
      SDisplayCursor cursor;
      setCursor( cursor );
    }
    break;

    case page_color:
    {
      SDisplayColor color;
      setColor( color );
    }
    break;

    case page_font:
    {
      SDisplayFont font;
      setFont( font );
    }
    break;

    case page_file:
    {
      SDisplayMisc misc;
      setFile( misc );
    }
    break;

    case page_misc:
    {
      SDisplayMisc misc;
      setMisc( misc );
    }
    break;
  }
}


void COptionDialog::slotApply( void )
{
  switch( activePage() )
  {
    case page_layout:
    {
      int index = mLayout.formatCombo->currentItem();
      mDisplayState.line.setLineSize(index, mLayout.lineSizeSpin->value());
      mDisplayState.line.setColumnSize(index, mLayout.columnSizeSpin->value());
      emit lineSizeChoice( mDisplayState.line );

      SDisplayLayout &l = mDisplayState.layout;
      l.lockLine = mLayout.lockLineCheck->isChecked();
      l.lockColumn = mLayout.lockColumnCheck->isChecked();
      l.leftSeparatorWidth = mLayout.leftSepWidthSpin->value();
      l.rightSeparatorWidth = mLayout.rightSepWidthSpin->value();
      l.separatorMarginWidth = mLayout.separatorSpin->value();
      l.edgeMarginWidth = mLayout.edgeSpin->value();
      l.leftSeparatorWidth = mLayout.leftSepWidthSpin->value();
      l.rightSeparatorWidth = mLayout.rightSepWidthSpin->value();
      l.columnCharSpace = mLayout.columnCheck->isChecked();
      l.columnSpacing = mLayout.columnSepSpin->value();

      if( mLayout.gridCombo->currentItem() == 0 )
      {
	l.horzGridWidth = l.vertGridWidth = 0;
      }
      else if( mLayout.gridCombo->currentItem() == 1 )
      {
	l.horzGridWidth = 0;
	l.vertGridWidth = 1;
      }
      else if( mLayout.gridCombo->currentItem() == 2 )
      {
	l.horzGridWidth = 1;
	l.vertGridWidth = 0;
      }
      else
      {
	l.horzGridWidth = l.vertGridWidth = 1;
      }

      emit layoutChoice( mDisplayState.layout );
    }
    break;


    case page_cursor:
    {
      SDisplayCursor &c = mDisplayState.cursor;      
      c.alwaysVisible    = mCursor.blinkCheck->isChecked();
      c.interval         = mCursor.blinkSpin->value();
      c.alwaysBlockShape = mCursor.blockCheck->isChecked();
      c.thickInsertShape = mCursor.thickCheck->isChecked();
      c.focusMode        = cursorFocusMode();
      emit cursorChoice( mDisplayState.cursor );
    }
    break;

    case page_color:
    {
      SDisplayColor &c = mDisplayState.color;
      c.useSystemColor = mColor.checkSystem->isChecked();
      c.textBg = mColor.colorList->color( SColorWidgets::FirstTextBg );
      c.secondTextBg = mColor.colorList->color( SColorWidgets::SecondTextBg );
      c.offsetBg = mColor.colorList->color( SColorWidgets::OffsetBg );
      c.inactiveBg = mColor.colorList->color( SColorWidgets::InactiveBg );
      c.markBg = mColor.colorList->color( SColorWidgets::MarkedBg );
      c.markFg = mColor.colorList->color( SColorWidgets::MarkedFg );
      c.primaryFg[0] = mColor.colorList->color( SColorWidgets::EvenColumnFg );
      c.primaryFg[1] = mColor.colorList->color( SColorWidgets::OddColumnFg );
      c.offsetFg  = mColor.colorList->color( SColorWidgets::OffsetFg );
      c.secondaryFg = mColor.colorList->color( SColorWidgets::SecondaryFg );
      c.cursorBg  = mColor.colorList->color( SColorWidgets::CursorBg );
      c.cursorFg  = mColor.colorList->color( SColorWidgets::CursorFg );
      c.leftSeparatorFg = mColor.colorList->color(SColorWidgets::SeparatorFg);
      c.rightSeparatorFg = mColor.colorList->color(SColorWidgets::SeparatorFg);
      c.nonPrintFg  = mColor.colorList->color( SColorWidgets::NonPrintFg );
      c.gridFg = mColor.colorList->color( SColorWidgets::GridFg );
      emit colorChoice( mDisplayState.color );
    }
    break;

    case page_font:
    {
      SDisplayFont &f = mDisplayState.font;
      f.useSystemFont = mFont.checkSystem->isChecked();
      f.localFont = mFont.chooser->font();

      const char *nonPrintText = mFont.nonPrintInput->text(); 
      if( nonPrintText == 0 || strlen(nonPrintText) == 0 )
      {
	f.nonPrintChar = ' ';
      }
      else
      {
	f.nonPrintChar = nonPrintText[0];
      }

      emit fontChoice( mDisplayState.font );
    }
    break;


    case page_file:
    case page_misc:
    {
      SDisplayMisc &m = mDisplayState.misc;
      m.undoLevel = mMisc.undoSpin->value();
      m.autoCopyToClipboard = mMisc.autoCheck->isChecked();
      m.insertMode = mMisc.insertCheck->isChecked();
      m.inputSound = mMisc.inputCheck->isChecked();
      m.confirmWrap = mMisc.confirmWrapCheck->isChecked();
      m.cursorJump = mMisc.cursorJumpCheck->isChecked();
      m.fatalSound = mMisc.fatalCheck->isChecked();
      m.confirmThreshold = mMisc.thresholdCheck->isChecked();
      m.thresholdValue = mMisc.thresholdSpin->value();

      m.openFile  = (SDisplayMisc::EOpenFile)mFile.openCombo->currentItem();
      m.gotoOnStartup = mFile.gotoOffsetCheck->isChecked();
      m.writeProtect = mFile.writeProtectCheck->isChecked();
      m.gotoOnReload = mFile.reloadOffsetCheck->isChecked();
      m.makeBackup = mFile.backupCheck->isChecked();
      m.discardRecent = mFile.discardRecentCheck->isChecked();

      emit miscChoice( mDisplayState.misc );
    }
    break;
  }

}

void COptionDialog::setLayout( SDisplayLayout &layout, SDisplayLine &line )
{
  mDisplayState.line = line;
  mDisplayState.layout = layout;

  slotModeSelectorChanged( mLayout.formatCombo->currentItem() );
  mLayout.lockLineCheck->setChecked( layout.lockLine );
  mLayout.lockColumnCheck->setChecked( layout.lockColumn );
  mLayout.leftSepWidthSpin->setValue( layout.leftSeparatorWidth );
  mLayout.rightSepWidthSpin->setValue( layout.rightSeparatorWidth );
  mLayout.separatorSpin->setValue( layout.separatorMarginWidth );
  mLayout.edgeSpin->setValue( layout.edgeMarginWidth );
  mLayout.leftSepWidthSpin->setValue( layout.leftSeparatorWidth );
  mLayout.rightSepWidthSpin->setValue( layout.rightSeparatorWidth );
  mLayout.columnCheck->setChecked( layout.columnCharSpace ); 
  slotColumnSepCheck( layout.columnCharSpace );
  mLayout.columnSepSpin->setValue( layout.columnSpacing );

  if( layout.horzGridWidth == 0 && layout.vertGridWidth == 0 )
  {
    mLayout.gridCombo->setCurrentItem(0);
  }
  else if( layout.horzGridWidth != 0 && layout.vertGridWidth != 0 )
  {
    mLayout.gridCombo->setCurrentItem(3);
  }
  else
  {
    mLayout.gridCombo->setCurrentItem( layout.vertGridWidth != 0 ? 1 : 2 );
  }

}


void COptionDialog::setCursor( SDisplayCursor &cursor )
{
  mDisplayState.cursor = cursor;
 
  mCursor.blinkCheck->setChecked( cursor.alwaysVisible );
  mCursor.blinkSpin->setValue( cursor.interval );
  mCursor.blockCheck->setChecked( cursor.alwaysBlockShape );
  mCursor.thickCheck->setChecked( cursor.thickInsertShape );
  bool state = cursor.focusMode == SDisplayCursor::stopBlinking ? true : false;
  mCursor.stopRadio->setChecked( state );
  state = cursor.focusMode == SDisplayCursor::hide ? true : false;
  mCursor.hideRadio->setChecked( state );
  state = cursor.focusMode == SDisplayCursor::ignore ? true : false;
  mCursor.nothingRadio->setChecked( state );
}


void COptionDialog::setColor( SDisplayColor &color )
{
  mDisplayState.color = color;
  mColor.colorList->setColor( SColorWidgets::FirstTextBg, color.textBg );
  mColor.colorList->setColor( SColorWidgets::SecondTextBg, color.secondTextBg);
  mColor.colorList->setColor( SColorWidgets::OffsetBg, color.offsetBg );
  mColor.colorList->setColor( SColorWidgets::InactiveBg, color.inactiveBg );
  mColor.colorList->setColor( SColorWidgets::MarkedBg, color.markBg );
  mColor.colorList->setColor( SColorWidgets::MarkedFg, color.markFg );
  mColor.colorList->setColor( SColorWidgets::EvenColumnFg, color.primaryFg[0]);
  mColor.colorList->setColor( SColorWidgets::OddColumnFg, color.primaryFg[1] );
  mColor.colorList->setColor( SColorWidgets::OffsetFg, color.offsetFg );
  mColor.colorList->setColor( SColorWidgets::SecondaryFg, color.secondaryFg );
  mColor.colorList->setColor( SColorWidgets::CursorBg, color.cursorBg );
  mColor.colorList->setColor( SColorWidgets::CursorFg, color.cursorFg );
  mColor.colorList->setColor(SColorWidgets::SeparatorFg,color.leftSeparatorFg);
  mColor.colorList->setColor( SColorWidgets::NonPrintFg, color.nonPrintFg );
  mColor.colorList->setColor( SColorWidgets::GridFg, color.gridFg );
}


void COptionDialog::setFont( SDisplayFont &font )
{
  mDisplayState.font = font;

  mFont.checkSystem->setChecked( font.useSystemFont );
  mFont.chooser->setFont( font.localFont, true );
  char buf[2] = { font.nonPrintChar, 0 };
  mFont.nonPrintInput->setText( buf );
}


void COptionDialog::setMisc( SDisplayMisc &misc )
{
  mDisplayState.misc.undoLevel           = misc.undoLevel;
  mDisplayState.misc.inputSound          = misc.inputSound;
  mDisplayState.misc.fatalSound          = misc.fatalSound;
  mDisplayState.misc.autoCopyToClipboard = misc.autoCopyToClipboard;
  mDisplayState.misc.insertMode          = misc.insertMode;
  mDisplayState.misc.confirmWrap         = misc.confirmWrap;
  mDisplayState.misc.cursorJump          = misc.cursorJump;
  mDisplayState.misc.confirmThreshold    = misc.confirmThreshold; 
  mDisplayState.misc.thresholdValue      = misc.thresholdValue;

  mMisc.undoSpin->setValue( misc.undoLevel );
  mMisc.inputCheck->setChecked( misc.inputSound );
  mMisc.fatalCheck->setChecked( misc.fatalSound );

  mMisc.autoCheck->setChecked( misc.autoCopyToClipboard );
  mMisc.insertCheck->setChecked( misc.insertMode );
  mMisc.confirmWrapCheck->setChecked( misc.confirmWrap );
  mMisc.cursorJumpCheck->setChecked( misc.cursorJump );

  mMisc.thresholdCheck->setChecked( misc.confirmThreshold );
  mMisc.thresholdSpin->setValue( misc.thresholdValue );
  slotThresholdConfirm();
}

void COptionDialog::setFile( SDisplayMisc &misc )
{
  mDisplayState.misc.openFile      = misc.openFile;
  mDisplayState.misc.gotoOnStartup = misc.gotoOnStartup;
  mDisplayState.misc.writeProtect  = misc.writeProtect;
  mDisplayState.misc.gotoOnReload  = misc.gotoOnReload;
  mDisplayState.misc.discardRecent = misc.discardRecent;
  mDisplayState.misc.makeBackup    = misc.makeBackup;

  mFile.openCombo->setCurrentItem( (uint)misc.openFile );
  mFile.gotoOffsetCheck->setChecked( misc.gotoOnStartup );
  mFile.writeProtectCheck->setChecked( misc.writeProtect );
  mFile.reloadOffsetCheck->setChecked( misc.gotoOnReload );
  mFile.discardRecentCheck->setChecked( misc.discardRecent );
  mFile.backupCheck->setChecked( misc.makeBackup );
}




void COptionDialog::setState( SDisplayState &state )
{
  setLayout( state.layout, state.line );
  setCursor( state.cursor );
  setColor( state.color );
  setFont( state.font );
  setFile( state.misc );
  setMisc( state.misc );
}


SDisplayCursor::EFocusMode COptionDialog::cursorFocusMode( void )
{
  if( mCursor.stopRadio->isChecked() == true )
  {
    return( SDisplayCursor::stopBlinking );
  }
  else if( mCursor.hideRadio->isChecked() == true )
  {
    return( SDisplayCursor::hide );
  }
  else
  {
    return( SDisplayCursor::ignore );
  }
}





CColorListBox::CColorListBox( QWidget *parent, const char *name, WFlags f )
  :QListBox( parent, name, f )
{
  connect( this, SIGNAL(selected(int)), this, SLOT(newColor(int)) );
}


void CColorListBox::setEnabled( bool state )
{
  if( state == isEnabled() )
  {
    return;
  }

  QListBox::setEnabled( state );
  for( uint i=0; i<count(); i++ )
  {
    updateItem( i );
  }
}


void CColorListBox::setColor( uint index, const QColor &color )
{
  if( index < count() )
  { 
    CColorListItem *colorItem = (CColorListItem*)item(index);
    colorItem->setColor(color);
    updateItem( index, false );
  }
} 


const QColor CColorListBox::color( uint index )
{
  if( index < count() )
  {
    CColorListItem *colorItem = (CColorListItem*)item(index);
    return( colorItem->color() );
  }
  else
  {
    return( black );
  }
}


void CColorListBox::newColor( int index ) 
{
  if( isEnabled() == false )
  {
    return;
  }

  if( (uint)index < count() )
  {
    QColor c = color( index );
    if( KColorDialog::getColor( c ) != QDialog::Rejected )
    {
      setColor( index, c );
    }
  }
}



CColorListItem::CColorListItem( const QString &text, const QColor &color )
  : QListBoxItem(), mColor( color ), mBoxWidth( 30 )
{
  setText( text );
}


const QColor &CColorListItem::color( void )
{
  return( mColor );
}


void CColorListItem::setColor( const QColor &color )
{
  mColor = color;
}


void CColorListItem::paint( QPainter *p )
{
  QFontMetrics fm = p->fontMetrics();
  int h = fm.height();

  p->drawText( mBoxWidth+3*2, fm.ascent() + fm.leading()/2, text() );

  p->setPen( black );
  p->drawRect( 3, 1, mBoxWidth, h-1 );
  p->fillRect( 4, 2, mBoxWidth-2, h-3, mColor );
}


int CColorListItem::height(const QListBox *lb ) const
{
  return( lb->fontMetrics().lineSpacing()+1 );
}


int CColorListItem::width(const QListBox *lb ) const
{
  return( mBoxWidth + lb->fontMetrics().width( text() ) + 6 );
}

