/*
 *   khexedit - Versatile hex editor
 *   Copyright (C) 1999  Espen Sand, espensa@online.no
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include <klocale.h>
#include "hexmanagerwidget.h"

CHexManagerWidget::CHexManagerWidget( QWidget *parent, const char *name,
				      EConversionPosition conversionPosition,
				      ETabPosition tabBarPosition )
  : QWidget( parent, name )
{
  mValid = false;

  mEditor = new CHexEditorWidget( this );
  if( mEditor == 0 ) { return; }

  mTabBar = new CTabBar( this );
  if( mTabBar == 0 ) { return; }
  mTabBar->hide();

  mConverter = new CHexToolWidget( this );
  if( mConverter == 0 ) { return; }

  connect( mEditor->view(), SIGNAL(fileName(const QString &)),
	   this, SLOT( addName(const QString &)));
  connect( mEditor->view(), SIGNAL( fileClosed(const QString &)),
	   this, SLOT( removeName(const QString &))); 
  connect( mEditor->view(),SIGNAL(fileRename(const QString &,const QString &)),
	   this, SLOT(changeName(const QString &,const QString &)));
  connect( mEditor->view(), SIGNAL( cursorChanged( SCursorState & ) ),
	   mConverter, SLOT( cursorChanged( SCursorState & ) ) );
  connect( mConverter, SIGNAL( closed(void) ), 
	   this, SIGNAL( conversionClosed(void) ) );
  connect( mTabBar, SIGNAL(selected(const QString &)), 
	   this, SLOT(open(const QString &)));

  mValid = true;
  setConversionVisibility( conversionPosition );
  setTabBarPosition( tabBarPosition );
}


CHexManagerWidget::~CHexManagerWidget( void )
{
  delete mEditor;
  delete mTabBar;
  delete mConverter;
}


void CHexManagerWidget::resizeEvent( QResizeEvent * )
{
  if( mValid == false ) { return; }

  int h = mTabBar->isVisible() ? mTabBar->sizeHint().height() : 0;
  if( mConversionPosition != InEditor )
  {
    if( mTabPosition == AboveEditor )
    {
      mTabBar->setGeometry( 0, 0, width(), h );
      mEditor->setGeometry( 0, h, width(), height()-h );
    }
    else
    {
      mEditor->setGeometry( 0, 0, width(), height()-h );
      mTabBar->setGeometry( 0, height()-h, width(), h );
    }
  }
  else
  {
    uint utilHeight = mConverter->minimumSize().height();
    if( mTabPosition == AboveEditor )
    {
      mTabBar->setGeometry( 0, 0, width(), h );
      mEditor->setGeometry( 0, h, width(), height() - utilHeight - h );
      mConverter->setGeometry( 0, height() - utilHeight, width(), utilHeight );
    }
    else
    {
      mEditor->setGeometry( 0, 0, width(), height() - utilHeight - h );
      mTabBar->setGeometry( 0, height() - utilHeight - h, width(), h );
      mConverter->setGeometry( 0, height() - utilHeight, width(), utilHeight );
    }
  }
}


void CHexManagerWidget::setConversionVisibility( EConversionPosition position )
{
  if( mValid == false )
  {
    return;
  }

  if( mConversionPosition == position )
  {
    if( mConversionPosition == InWindow )
    {
      mConverter->raise();
    }
    return;
  }

  mConversionPosition = position;
  if( mConversionPosition == Hide )
  {
    mConverter->hide();
  }
  else if( mConversionPosition == InWindow )
  {
    QPoint point = mapToGlobal( QPoint(0,0) );
    QRect  rect  = geometry();
    QPoint p;
    
    p.setX(point.x() + rect.width()/2 - mConverter->minimumSize().width()/2);
    p.setY(point.y() + rect.height()/2 - mConverter->minimumSize().height()/2);
    mConverter->resize( mConverter->minimumSize() );
    mConverter->recreate( 0, WStyle_Customize | WStyle_DialogBorder, p, true );
    mConverter->setCaption(kapp->makeStdCaption(i18n("Conversion")));
  }
  else
  {
    mConversionPosition = InEditor;
    uint utilHeight = mConverter->minimumSize().height();
    QPoint p( 0, height() - utilHeight );
    mConverter->recreate( this, 0, p, true );
  }

  resizeEvent(0);

}


void CHexManagerWidget::setTabBarPosition( ETabPosition position )
{
  mTabPosition = position;
  if( mTabPosition != HideTab && mTabBar->count() > 0 )
  {
    mTabBar->show();
  }
  else
  {
    mTabBar->hide();
  }
  resizeEvent(0);
}


void CHexManagerWidget::addName( const QString &name )
{
  if( name.isEmpty() == true )
  {
    return;
  }

  mTabBar->addName( name );
  if( mTabBar->isVisible() == false && mTabPosition != HideTab )
  {
    mTabBar->show();
    resizeEvent(0);
  }
}


void CHexManagerWidget::removeName( const QString &name )
{
  mTabBar->removeName( name );
  if( mTabBar->isVisible() == true && mTabBar->count() == 0 )
  {
    mTabBar->hide();
    resizeEvent(0);
  }
}


void CHexManagerWidget::changeName( const QString &curName, 
				    const QString &newName )
{
  mTabBar->changeName( curName, newName );
}


void CHexManagerWidget::open( const QString &name )
{
  mEditor->open( name, false, 0 );
}


CTabBar::CTabBar( QWidget *parent, char *name )
  :QTabBar( parent, name )
{
  connect( this, SIGNAL(selected(int)), this, SLOT(slotSelected(int)) );
}


void CTabBar::addName( const QString &name )
{
  QString n( name.right(name.length()-name.findRev('/')-1) );

  QTab *t = find( n );
  if( t == 0 )
  {
    t = new QTab();
    t->label = n;
    int id = addTab( t );
    mFileList.append( CFileKey(name,id) );
  }
  setCurrentTab(t);
}


void CTabBar::removeName( const QString &name )
{
  QString n( name.right(name.length()-name.findRev('/')-1) );
  QTab *t = find(n);
  if( t == 0 )
  {
    return;
  }

  QValueList<CFileKey>::Iterator it;
  for( it = mFileList.begin(); it != mFileList.end(); ++it )
  {
    if( (*it).id() == t->id )
    {
      mFileList.remove(it);
      removeTab(t); 
      layoutTabs();
      break;
    }
  }
}


void CTabBar::changeName( const QString &curName, const QString &newName )
{
  QString n( curName.right(curName.length()-curName.findRev('/')-1) );
  QTab *t = find(n);
  if( t == 0 )
  {
    return;
  }

  QValueList<CFileKey>::Iterator it;
  for( it = mFileList.begin(); it != mFileList.end(); ++it )
  {
    if( (*it).id() == t->id )
    {
      QString m( newName.right(newName.length()-newName.findRev('/')-1) );
      t->label = m;

      mFileList.remove(it);
      mFileList.append( CFileKey(newName,t->id) );
      layoutTabs();
      update(); // Seems to be necessary
      break;
    }
  }
}


QTab *CTabBar::find( const QString &name )
{
  QList<QTab> &list = *tabList();
  for( QTab *t = list.first(); t != 0; t = list.next() )
  {
    if( t->label == name )
    {
      return( t );
    }
  }

  return( 0 );
}


int CTabBar::count( void )
{
  return( tabList()->count() );
}


void CTabBar::slotSelected( int id )
{
  QValueList<CFileKey>::Iterator it;
  for( it = mFileList.begin(); it != mFileList.end(); ++it )
  {
    if( (*it).id() == id )
    {
      emit selected( (*it).filename() );
    }
  }
}




