/*
 *   khexedit - Versatile hex editor
 *   Copyright (C) 1999  Espen Sand, espensa@online.no
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <iostream.h>

#include <qtooltip.h> 

#include <kapp.h>
#include <kiconloader.h>
#include <kglobal.h>
#include <klocale.h>
#include <kmenubar.h>
#include <kmessagebox.h>
#include <kstdaccel.h>
#include <ktoolbarbutton.h>

#include "dialog.h"
#include "hexerror.h"
#include "toplevel.h"


QList<KHexEdit> KHexEdit::mWindowList;

KHexEdit::KHexEdit( void )
   : KTMainWindow("toplevel"), mStartupOffset(0)
{
  mWindowList.append( this );
  mValid = false;

  mAboutKHexEdit = 0;

  KIconLoader &loader = *KGlobal::iconLoader();
  QPixmap pixmap;

  //
  // Create main widget and register it.
  //
  mManager = new CHexManagerWidget( this, "manager" );
  if( mManager == 0 ) { return; }
  setView( mManager );


  connect( textView(), SIGNAL( cursorChanged( SCursorState & ) ),
	   this, SLOT( cursorChanged( SCursorState & ) ) );
  connect( textView(), SIGNAL( editMode( CHexBuffer::EEditMode ) ),
	   this, SLOT( editMode( CHexBuffer::EEditMode ) ) );
  connect( textView(), SIGNAL( encodingChanged( const SEncodeState &)),
	   this, SLOT( encodingChanged( const SEncodeState & )) );
  connect( textView(), SIGNAL( textWidth( uint ) ),
	   this, SLOT( textWidth( uint ) ) );
  connect( textView(), SIGNAL( fileState( SFileState & ) ),
	   this, SLOT( fileState( SFileState & ) ) );
  connect( textView(), SIGNAL( layoutChanged( const SDisplayLayout & ) ),
	   this, SLOT( layoutChanged( const SDisplayLayout & ) ) );
  connect( textView(), SIGNAL( inputModeChanged( const SDisplayInputMode & ) ),
	   this, SLOT( inputModeChanged( const SDisplayInputMode & ) ) );
  connect( textView(), SIGNAL( bookmarkChanged( QList<SCursorOffset> &)),
	   this, SLOT( bookmarkChanged( QList<SCursorOffset> & ) ) );
  connect( textView(), SIGNAL( fileName( const QString & ) ),
	   this, SLOT( fileActive( const QString & ) ) );
  connect( textView(), SIGNAL( fileRename( const QString &, const QString & )),
	   this, SLOT( fileRename( const QString &, const QString & ) ) );
  connect( textView(), SIGNAL( fileClosed( const QString & ) ),
	   this, SLOT( fileClosed( const QString & ) ) ); 
  connect( editor(), SIGNAL( errorLoadFile( const QString & ) ),
	   this, SLOT( removeRecentFile( const QString & ) ) );
  connect( editor(), SIGNAL( operationChanged( bool ) ),
	   this, SLOT( operationChanged( bool ) ) );
  connect( editor(), SIGNAL( removeRecentFiles() ),
	   this, SLOT( removeRecentFiles() ) );
  connect( mManager, SIGNAL( conversionClosed() ), 
	   this, SLOT(conversionClosed()) );
 
  //
  // I use an eventfilter to catch all drag/drop events for the 
  // area outside the editor window. It will only accept QUriDrag.
  // (The editor window will catch QUriDrag, QTextDrag and QHexDrag)
  //
  installEventFilter( this );
  setAcceptDrops(true);

  //
  // Make the editor popup menu.
  //
  mEditorPopup = new QPopupMenu;
  if( mEditorPopup == 0 ) { return; }
  connect( mEditorPopup, SIGNAL(activated (int)), SLOT( viewMenuCB (int) ));
  mBookmarkPopup = new QPopupMenu;
  if( mBookmarkPopup == 0 ) { return; }
  connect( mBookmarkPopup, SIGNAL(activated (int)),SLOT(bookmarkPopupCB(int)));


  mEditorPopup->setCheckable( TRUE );
  mEditorPopup->insertItem( i18n("&Open..."), editor(), SLOT(open()) );
  mEditorPopup->insertItem( i18n("&Save"), editor(), SLOT(save()) );
  mEditorPopup->insertItem( i18n("Save &as..."), editor(), SLOT(saveAs()) );
  mEditorPopup->insertSeparator();
  mEditorPopup->insertItem( i18n("&Bookmarks"), mBookmarkPopup, 0, 4 );
  mEditorPopup->insertSeparator();
  mPopHexMB = mEditorPopup->insertItem( i18n("&Hexadecimal"),view_hexadecimal);
  mPopOctalMB = mEditorPopup->insertItem( i18n("&Octal"), view_octal );
  mPopBinaryMB = mEditorPopup->insertItem( i18n("&Binary"), view_binary );
  mPopTextOnlyMB = mEditorPopup->insertItem( i18n("&Text only"), view_text );
  mEditorPopup->insertSeparator();
  mPopReadOnlyMB = mEditorPopup->insertItem( i18n("&Read only"),
					     permission_readonly );
  mPopResizeMB = mEditorPopup->insertItem( i18n("&Allow resize"),
					   permission_allowResize );
  textView()->setPopupMenu( mEditorPopup );


  //
  // Make the menu bar entries.
  //
  mRecent = new QPopupMenu;
  if( mRecent == 0 ) { return; }
  connect( mRecent, SIGNAL(activated(int)), SLOT(recentFile(int)) );

  mFilePermission = new QPopupMenu;
  if( mFilePermission == 0 ) { return; }

  mFile = new QPopupMenu( this );
  if( mFile == 0 ) { return; }

  mEditSpecial = new QPopupMenu;
  if( mEditSpecial == 0 ) { return; }

  mEdit = new QPopupMenu;
  if( mEdit == 0 ) { return; }

  mEncoding = new QPopupMenu;
  if( mEncoding == 0 ) { return; }

  mView = new QPopupMenu;
  if( mView == 0 ) { return; }

  mBookmark = new QPopupMenu;
  if( mBookmark == 0 ) { return; }

  mDocuments = new QPopupMenu;
  if( mDocuments == 0 ) { return; }
  connect( mDocuments, SIGNAL(activated(int)), SLOT(documentMenuCB(int)) );

  mTools = new QPopupMenu;
  if( mTools == 0 ) { return; }

  mConversion = new QPopupMenu;
  if( mConversion == 0 ) { return; }

  mDocumentTabs = new QPopupMenu;
  if( mDocumentTabs == 0 ) { return; }

  mOptions = new QPopupMenu;
  if( mOptions == 0 ) { return; }

  mFilePermission->setCheckable( TRUE );
  mPermissionReadOnlyMB = mFilePermission->insertItem( i18n("&Read only"), 
		      permission_readonly );
  mPermissionResizeMB = mFilePermission->insertItem( i18n("&Allow resize"), 
		      permission_allowResize );
  connect( mFilePermission, SIGNAL(activated (int)), SLOT( viewMenuCB (int) ));

  mEncoding->setCheckable( TRUE );
  mEncodingDefaultMB = mEncoding->insertItem( i18n("&Default"), 
		       encoding_default );
  mEncodingUsAsciiMB = mEncoding->insertItem( i18n("US-&ASCII (7 bit)"),
		       encoding_us_ascii );
  mEncodingEbcdicMB  = mEncoding->insertItem( i18n("&EBCDIC"), 
		       encoding_ebcdic );
  connect( mEncoding, SIGNAL(activated (int)), SLOT( encodingMenuCB (int) ));
  mEncoding->insertSeparator();
  mEncoding->insertItem( i18n("&User Defined..."),
			 editor(), SLOT(encoding()) );

  mFileNewDocumentMB = mFile->insertItem( i18n("&New Document"), 
		     editor(), SLOT(newFile()), KStdAccel::openNew() );
  mFileOpenMB = mFile->insertItem( i18n("&Open..."), 
		     editor(), SLOT(open()), KStdAccel::open() );
  mFile->insertItem( i18n("&Insert..."), 
		     editor(), SLOT(insertFile()), CTRL+Key_I );
  mFile->insertItem( i18n("Open R&ecent"), mRecent );
  mFileReloadMB = mFile->insertItem( i18n("&Reload"), 
		     editor(), SLOT(reload()) );
  mFile->insertSeparator();
  mFileSaveMB = mFile->insertItem( i18n("&Save"), 
		     editor(), SLOT(save()), KStdAccel::save() );
  mFile->insertItem( i18n("Save &as..."), 
		     editor(), SLOT(saveAs()), CTRL+Key_A );
  mFile->insertItem( i18n("&Close Document"), 
		     editor(), SLOT(close()), KStdAccel::close() );
  mFile->insertSeparator();
  mFilePrintMB = mFile->insertItem( i18n("&Print Document"), 
		     editor(), SLOT(print()), KStdAccel::print() );
  mFile->insertItem( i18n("E&xport..."), 
		     editor(), SLOT(exportDialog()) );
  mFile->insertSeparator();
  mFileCancelOperationMB = mFile->insertItem( i18n("&Cancel Operation"), 
		     editor(), SLOT(stop()) );
  mFile->insertItem( i18n("Per&missions"), mFilePermission );
  mFile->insertSeparator();
  mFile->insertItem( i18n("N&ew Window"), 
		     this, SLOT(newWindow()) );
  mFile->insertItem( i18n("Close &Window"), 
		     this, SLOT(closeWindow()) );
  mFile->insertSeparator();
  mFileQuitMB = mFile->insertItem( i18n("&Quit"), 
		     this, SLOT(closeProgram()), KStdAccel::quit() );

  mEditUndoMB = mEdit->insertItem( i18n("Und&o"), 
		     editor(), SLOT(undo()), KStdAccel::undo() );
  mEditRedoMB = mEdit->insertItem( i18n("Re&do"), 
		     editor(), SLOT(redo()), KStdAccel::redo() );
  mEdit->insertSeparator();
  mEditCutMB = mEdit->insertItem( i18n("C&ut"),
		     editor(), SLOT(cut()), KStdAccel::cut() );
  mEditCopyMB = mEdit->insertItem( i18n("&Copy"),
		     editor(), SLOT(copy()), KStdAccel::copy() );
  mEditPasteMB = mEdit->insertItem( i18n("&Paste"),
		     editor(), SLOT(paste()), KStdAccel::paste() );
  mEdit->insertItem( i18n("Spec&ial"), mEditSpecial );
  mEdit->insertSeparator();
  mEdit->insertItem( i18n("Select &All"), 
		     editor(), SLOT(selectAll()) );
  mEdit->insertItem( i18n("&Unselect"), 
		     editor(), SLOT(unselect()) );
  mEdit->insertSeparator();
  mEditFindMB = mEdit->insertItem( i18n("&Find..."),
		     editor(), SLOT(find()), KStdAccel::find() );
  mEdit->insertItem( i18n("Find &Next"),
		     editor(), SLOT(findAgain()), KStdAccel::findNext() );
  mEdit->insertItem( i18n("Find Pre&vious..."),
		     editor(), SLOT(findPrevious()), KStdAccel::findPrev() );
  mEdit->insertItem( i18n("&Replace..."),
		     editor(), SLOT(replace()), KStdAccel::replace() );
  mEdit->insertSeparator();
  mEdit->insertItem( i18n("&Goto Offset..."),
		     editor(), SLOT(gotoOffset()), CTRL+Key_G );
  mEdit->insertItem( i18n("&Insert Pattern..."),
		     editor(), SLOT(insertPattern()), KStdAccel::insert() );

  mEditSpecialCopyText1MB = mEditSpecial->insertItem( 
			    i18n("Copy as text (&primary field)"),
			    editor(), SLOT(copyPrimaryFormatted()) );
  mEditSpecialCopyText2MB = mEditSpecial->insertItem( 
			    i18n("Copy as text (&all fields)"),
			    editor(), SLOT(copyAllFormatted()) );
  mEditSpecial->insertSeparator();
  mEditSpecial->insertItem( i18n("Paste into new &file"),
			    editor(), SLOT(pasteNewFile()) );
  mEditSpecial->insertItem( i18n("Paste into new &window"),
			    this, SLOT(pasteNewWindow()) );

  mView->setCheckable( TRUE );
  mViewHexMB      = mView->insertItem( i18n("&Hexadecimal"), 
		     view_hexadecimal );
  mViewOctalMB    = mView->insertItem( i18n("&Octal"), 
		     view_octal );
  mViewBinaryMB   = mView->insertItem( i18n("&Binary"), 
		     view_binary );
  mViewTextOnlyMB = mView->insertItem( i18n("&Text only"), 
		     view_text );
  mView->insertSeparator();
  mViewOffsetColumnMB = mView->insertItem( i18n("Show o&ffset column"), 
		     view_offset_area );
  mViewTextAreaMB = mView->insertItem( i18n("Show t&ext field"), 
		     view_text_area );
  mView->insertSeparator();
  mViewOffsetAsDecimalMB = mView->insertItem( i18n("Off&set as decimal"), 
		     view_offset_as_decimal );
  mView->insertSeparator();
  mViewBigCaseDataMB = mView->insertItem( i18n("&Upper case (data)"), 
					  view_case_data );
  mViewBigCaseOffsetMB = mView->insertItem( i18n("U&pper case (offset)"),
					    view_case_offset );

  mView->insertSeparator();
  mView->insertItem( i18n("&Document Encoding"), mEncoding );

  connect( mView, SIGNAL(activated (int)), SLOT( viewMenuCB (int) ));

  mTools->insertItem( i18n("&Extract Strings..."),
		      editor(), SLOT(strings()) );
  mTools->insertItem( i18n("&Record Viewer..."),
		      editor(), SLOT(recordView()) );
  mTools->insertItem( i18n("&Binary Filter..."),
		      editor(), SLOT(filter()) );
  mTools->insertItem( i18n("&Character Table..."),
		      editor(), SLOT(chart()) );
  mTools->insertItem( i18n("Select &Indicator..."),
		      editor(), SLOT(selectIndicator()) );
  mTools->insertSeparator();
  mTools->insertItem( i18n("&Statistics..."),
		      editor(), SLOT(statistics()) );

  mBookmark->insertItem( i18n("&Add Bookmark"), 0, 0 );
  mBookmark->setAccel( CTRL+Key_B, 0 );
  mBookmark->insertItem( i18n("&Replace Bookmark"), 1, 1 );
  mBookmark->setAccel( CTRL+Key_E, 1 );
  mBookmark->insertItem( i18n("R&emove Bookmark"), 2, 2 );
  mBookmark->setAccel( CTRL+Key_U, 2 );
  mBookmark->insertItem( i18n("Re&move All"), 3, 3 );
  connect( mBookmark, SIGNAL(activated (int)), SLOT( bookmarkMenuCB (int) ));

  mConversion->setCheckable( true );
  mConversionHideMB   = mConversion->insertItem(i18n("&Hide"), 
			conversion_hide );
  mConversionWindowMB = mConversion->insertItem(i18n("Show In &Own Window"), 
			conversion_window );
  mConversionEditorMB = mConversion->insertItem(i18n("&Embed In Main Window"),
			conversion_editor );
  connect( mConversion, SIGNAL(activated (int)), SLOT( optionMenuCB(int) ));
  
  mDocumentTabs->setCheckable( true );
  mDocumentTabsHideMB  = mDocumentTabs->insertItem(i18n("&Hide"), 
			 documenttabs_hide );
  mDocumentTabsAboveMB = mDocumentTabs->insertItem(i18n("&Above editor"), 
			 documenttabs_above );
  mDocumentTabsBelowMB = mDocumentTabs->insertItem(i18n("&Below editor"),
			 documenttabs_below );
  connect( mDocumentTabs, SIGNAL(activated (int)), SLOT( optionMenuCB(int) ));

  mOptions->setCheckable( true );
  mOptionStatusBarMB  = mOptions->insertItem( i18n("Show &Statusbar"), 
		        option_statusbar );
  mOptionToolBarMB    = mOptions->insertItem( i18n("Show &Toolbar"), 
		        option_toolbar );
  mOptions->insertItem( i18n("&Document Tabs"), mDocumentTabs );
  mOptions->insertItem( i18n("&Conversion Field"), mConversion );
  mOptions->insertSeparator();
  mOptions->insertItem( i18n("&Preferences..."),
		        option_edit );
  mOptions->insertItem( i18n("&Favorites..."),
		        option_favorites );
  mOptions->insertSeparator();
  mOptions->insertItem( i18n("S&ave Configuration"), this,
			SLOT(writeConfiguration()) );
  connect( mOptions, SIGNAL(activated (int)), SLOT( optionMenuCB (int) ));

  /*

  mOptions->setCheckable( true );
  mOptions->insertItem( i18n("&Customize..."),
		        option_edit );
  mOptions->insertItem( i18n("&Favorites..."),
		        option_favorites );
  mOptions->insertSeparator();
  mOptions->insertItem( i18n("Conversion F&ield"), mConversion );
  mOptions->insertItem( i18n("&Document tabs"), mDocumentTabs );
  mOptionToolBarMB    = mOptions->insertItem( i18n("Show &Toolbar"), 
		        option_toolbar );
  mOptionStatusBarMB  = mOptions->insertItem( i18n("Show &Statusbar"), 
		        option_statusbar );


  mOptions->insertSeparator();
  mOptions->insertItem( i18n("S&ave Configuration"), this,
			SLOT(writeConfiguration()) );
  connect( mOptions, SIGNAL(activated (int)), SLOT( optionMenuCB (int) ));
  */


  //
  // HACK 1999-05-15-ES (KDE-1.1, KDE-1.1.1, KDE-2-alpha)
  // Here comes a little hack. The 'Documents' menu does not contain any 
  // entries by default. The entries will be added later and shall contain
  // an accelerator (Ctrl+<number=1-9>). The menu entry is created once a
  // document is installed, but without the hack, no accelerator is created.
  // Experimenting has shown that the accelaretor becomes visible if the 
  // menubar goes from "top" to "floating". If I remove all entries, and then
  // add one again it will get the accelerator. So that is what I do here.
  // I add one entry and remove it once the 'Document' menu is inserted.
  // (See addDocument() below) 
  //
  //mDocuments->insertItem( "dummy", 0, 0 );
  //mDocuments->setAccel( CTRL+Key_1, 0 );


  enableMenuIcon( true );



  //  
  // Define menu entries in the menu bar.
  //
  menuBar()->insertItem( i18n("&File"), mFile );
  menuBar()->insertItem( i18n("&Edit"), mEdit );
  menuBar()->insertItem( i18n("&View"), mView );
  menuBar()->insertItem( i18n("&Bookmarks"), mBookmark );
  menuBar()->insertItem( i18n("&Tools"), mTools );
  menuBar()->insertItem( i18n("&Documents"), mDocuments );
  menuBar()->insertItem( i18n("&Options"), mOptions );
  menuBar()->insertSeparator();
  menuBar()->insertItem( i18n("&Help"), helpMenu() );

  //
  // The dummy has now served its purpose (see above)
  //
  //mDocuments->removeItemAt( 0 );

  //
  // Make the toolbar
  //

  mUrlLabel = new CUrlLabel( toolBar(0) );
  if( mUrlLabel == 0 ) { return; }
  mUrlLabel->setPixmap( loader.loadIcon( "hexdrag" ) );
  mUrlLabel->setDragMask( loader.loadIcon( "hexmask" ) );

  QToolTip::add( mUrlLabel, i18n("Drag document") );
  toolBar(0)->insertWidget( -1, 22, mUrlLabel );

  pixmap = loader.loadIcon( "filenew" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(newFile()), true,
			    i18n("New document") );
  pixmap = loader.loadIcon( "fileopen" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(open()), true,
			    i18n("Open document") );
  pixmap = loader.loadIcon( "reload" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(reload()), true,
			    i18n("Reload document") );
  pixmap = loader.loadIcon( "filefloppy" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(save()), true,
			    i18n("Save document") );
  pixmap = loader.loadIcon( "fileprint" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(print()), true,
			    i18n("Print document") );
  pixmap = loader.loadIcon( "search" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(find()), true,
			    i18n("Find") );
  pixmap = loader.loadIcon( "next" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(findNext()), true,
			    i18n("Find next") );
  pixmap = loader.loadIcon( "prev" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(findPrevious()), true,
			    i18n("Find previous") );
  pixmap = loader.loadIcon( "editcut" );
  toolBar(0)->insertButton( pixmap, toolbar_Cut, SIGNAL(clicked()),
			    editor(), SLOT(cut()), true,
			    i18n("Cut") );
  pixmap = loader.loadIcon( "editcopy" );
  toolBar(0)->insertButton( pixmap, toolbar_Copy, SIGNAL(clicked()),
			    editor(), SLOT(copy()), true,
			    i18n("Copy") );
  pixmap = loader.loadIcon( "editpaste" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(paste()), true,
			    i18n("Paste") );
  pixmap = loader.loadIcon( "undo" );
  toolBar(0)->insertButton( pixmap, toolbar_Undo, SIGNAL(clicked()),
			    editor(), SLOT(undo()), true,
			    i18n("Undo") );
  pixmap = loader.loadIcon( "redo" );
  toolBar(0)->insertButton( pixmap, toolbar_Redo, SIGNAL(clicked()),
			    editor(), SLOT(redo()), true,
			    i18n("Redo") );
  /*
  pixmap = loader.loadIcon( "flag" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(benchmark()), true,
			    i18n("Benchmark") );
  */

  pixmap = loader.loadIcon( "lock");
  toolBar(0)->insertButton( pixmap, toolbar_rw, SIGNAL(clicked()),
			    editor(), SLOT(toggleWriteProtection()), true,
			    i18n("Toggle write protection") );
  toolBar(0)->alignItemRight( toolbar_rw );


  pixmap = loader.loadIcon( "stop" );
  toolBar(0)->insertButton( pixmap, toolbar_cancelOperation, SIGNAL(clicked()),
			    editor(), SLOT(stop()), true,
			    i18n("Cancel operation") );

  pixmap = loader.loadIcon( "help" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    this, SLOT(appHelpActivated()), true,
			    i18n("Help") );

  //
  // Prepare the status bar. It will display (from right) the offset
  // in the file, Save state ( a pixmap), Insert state, general messages
  //

  statusBar()->setInsertOrder( KStatusBar::RightToLeft );
  statusBar()->insertItem( i18n("RW"), 0 );
  statusBar()->insertItem( i18n("Bit: F"), 1 );
  statusBar()->insertItem( i18n("Offset: FFFFFFFFFF"), 2 );
  statusBar()->insertItem( i18n("Size: FFFFFFFFFF"), 3 );
  statusBar()->insertItem( i18n("OVR"), 4 );
  statusBar()->insertItem( i18n("M"), 5 );
  statusBar()->insertItem( i18n("Default encoding"), 6 );

  CStatusBarProgress *progess = new CStatusBarProgress( statusBar() );
  if( progess == 0 ) { return; }
  statusBar()->insertWidget( progess, 10, 7 );
 
  connect( editor(), SIGNAL( setProgress( int ) ),
	   progess, SLOT( setValue( int ) ) );
  connect( editor(), SIGNAL( setProgress( int, int ) ),
	   progess, SLOT( setValue( int, int ) ) );
  connect( editor(), SIGNAL( enableProgressText( bool ) ),
	   progess, SLOT( setTextEnabled( bool ) ) );
  connect( editor(), SIGNAL( setProgressText( const QString & ) ),
	   progess, SLOT( setText( const QString & ) ) );
  
  statusBar()->setAlignment( 0, AlignCenter );
  statusBar()->setAlignment( 4, AlignCenter );
  statusBar()->setAlignment( 5, AlignCenter );
  statusBar()->changeItem( i18n("Bit:"), 1 );
  statusBar()->changeItem( i18n("Offset:"), 2 );
  statusBar()->changeItem( i18n("Size:"), 3 );
  statusBar()->changeItem( "", 5 );
  statusBar()->changeItem( "", 6 );

  connect( statusBar(), SIGNAL(pressed( int )), 
	   this, SLOT(statusBarPressed( int )) );
  connect( progess, SIGNAL(pressed()), 
	   editor(), SLOT(stop()) );

  //
  // Read configuration from file. We will only allow files to 
  // be automatically opened if this in the first window.
  //
  readConfiguration();
  initialize( mWindowList.count() == 1 ? true : false );
  setStartupGeometry();

  mValid = true;
}


KHexEdit::~KHexEdit( void )
{
  delete mAboutKHexEdit;
  delete mManager;
}


void KHexEdit::enableMenuIcon( bool /*state*/ )
{
  KIconLoader &loader = *KGlobal::iconLoader();
  QPixmap pixmap;
  QString text;
    
  pixmap = loader.loadIcon( "filenew" );
  text = mFile->text( mFileNewDocumentMB );
  mFile->changeItem( mFileNewDocumentMB, QIconSet(pixmap), text );

  pixmap = loader.loadIcon( "fileopen" );
  text = mFile->text( mFileOpenMB );
  mFile->changeItem( mFileOpenMB, QIconSet(pixmap), text );

  pixmap = loader.loadIcon( "reload" );
  text = mFile->text( mFileReloadMB );
  mFile->changeItem( mFileReloadMB, QIconSet(pixmap), text );

  pixmap = loader.loadIcon( "filefloppy" );
  text = mFile->text( mFileSaveMB );
  mFile->changeItem( mFileSaveMB, QIconSet(pixmap), text );

  pixmap = loader.loadIcon( "fileprint" );
  text = mFile->text( mFilePrintMB );
  mFile->changeItem( mFilePrintMB, QIconSet(pixmap), text );

  pixmap = loader.loadIcon( "exit" );
  text = mFile->text( mFileQuitMB );
  mFile->changeItem( mFileQuitMB, QIconSet(pixmap), text );


  pixmap = loader.loadIcon( "undo" );
  text = mEdit->text( mEditUndoMB );
  mEdit->changeItem( mEditUndoMB, QIconSet(pixmap), text );

  pixmap = loader.loadIcon( "redo" );
  text = mEdit->text( mEditRedoMB );
  mEdit->changeItem( mEditRedoMB, QIconSet(pixmap), text );

  pixmap = loader.loadIcon( "editcopy" );
  text = mEdit->text( mEditCopyMB );
  mEdit->changeItem( mEditCopyMB, QIconSet(pixmap), text );

  pixmap = loader.loadIcon( "editpaste" );
  text = mEdit->text( mEditPasteMB );
  mEdit->changeItem( mEditPasteMB, QIconSet(pixmap), text );

  pixmap = loader.loadIcon( "editcut" );
  text = mEdit->text( mEditCutMB );
  mEdit->changeItem( mEditCutMB, QIconSet(pixmap), text );

  pixmap = loader.loadIcon( "search" );
  text = mEdit->text( mEditFindMB );
  mEdit->changeItem( mEditFindMB, QIconSet(pixmap), text );
}


void KHexEdit::removeRecentFiles( void )
{
  mRecent->clear();
}


void KHexEdit::initialize( bool openFiles )
{
  mUndoState = UNDO_OK;   // Reset in function below
  setUndoState( 0 ); 

  mSelectionValid = true; // Reset in function below
  setSelectionState( false );

  operationChanged( false );

  editor()->initialize();
 
  mOptions->setItemChecked( mOptionToolBarMB, mShowToolBar );
  if( mShowToolBar == true )
  {
    enableToolBar(KToolBar::Show);
  }
  else
  {
    enableToolBar(KToolBar::Hide);
  }

  mOptions->setItemChecked( mOptionStatusBarMB, mShowStatusBar );
  if( mShowStatusBar == true )
  {
    enableStatusBar(KStatusBar::Show);
  }
  else
  {
    enableStatusBar(KStatusBar::Hide);
  }

  if( mRecent->count() > 0 && openFiles == true )
  {
    if( editor()->openFile() == SDisplayMisc::allRecent )
    {
      for( uint i=0; i < mRecent->count(); i++ )
      {
	mStartupFileList.append( mRecent->text(i) );
	if( editor()->gotoStartupOffset() == true &&
	    mRecentOffsetList.count() > i )
	{
	  mStartupOffsetList.append( *mRecentOffsetList.at(i) );
	}
	else
	{
	  mStartupOffsetList.append( QString("0") );
	}
      }
    }
    else if( editor()->openFile() == SDisplayMisc::mostRecent )
    {
      mStartupFileList.append( mRecent->text(0) );
      if( editor()->gotoStartupOffset() == true &&
	  mRecentOffsetList.count() > 0 )
      {
	mStartupOffsetList.append( *mRecentOffsetList.at(0) );
      }
      else
      {
	mStartupOffsetList.append( QString("0") );
      }
    }
  }

  //
  // Open file(s) after editor has been displayed. 
  // Main reason: If anything goes wrong and an error dialog box
  // is displayed, then the editor will not be visible until the error
  // is confirmed and the (modal) dialog is closed.
  //
  startTimer( 100 );
}


void KHexEdit::timerEvent( QTimerEvent */*e*/ )
{
  killTimers();
  open( mStartupFileList, mStartupOffsetList );
  mStartupFileList.clear();
  mStartupOffsetList.clear();
}



void KHexEdit::setConversionVisibility( 
  CHexManagerWidget::EConversionPosition position )
{
  switch( position )
  {
    case CHexManagerWidget::Hide:
      mConversion->setItemChecked( mConversionHideMB, true );
      mConversion->setItemChecked( mConversionWindowMB, false );
      mConversion->setItemChecked( mConversionEditorMB, false );
      mManager->setConversionVisibility( CHexManagerWidget::Hide );
    break;

    case CHexManagerWidget::InWindow:
      mConversion->setItemChecked( mConversionHideMB, false );
      mConversion->setItemChecked( mConversionWindowMB, true );
      mConversion->setItemChecked( mConversionEditorMB, false );
      mManager->setConversionVisibility( CHexManagerWidget::InWindow );
    break;

    case CHexManagerWidget::InEditor:
    default:
      mConversion->setItemChecked( mConversionHideMB, false );
      mConversion->setItemChecked( mConversionWindowMB, false );
      mConversion->setItemChecked( mConversionEditorMB, true );
      mManager->setConversionVisibility( CHexManagerWidget::InEditor );
    break;
  }
}



void KHexEdit::setDocumentTabBarVisibility( 
  CHexManagerWidget::ETabPosition position )
{
  switch( position )
  {
    case CHexManagerWidget::HideTab:
      mDocumentTabs->setItemChecked( mDocumentTabsHideMB, true );
      mDocumentTabs->setItemChecked( mDocumentTabsAboveMB, false );
      mDocumentTabs->setItemChecked( mDocumentTabsBelowMB, false );
      mManager->setTabBarPosition( CHexManagerWidget::HideTab );
    break;

    case CHexManagerWidget::BelowEditor:
      mDocumentTabs->setItemChecked( mDocumentTabsHideMB, false );
      mDocumentTabs->setItemChecked( mDocumentTabsAboveMB, false );
      mDocumentTabs->setItemChecked( mDocumentTabsBelowMB, true );
      mManager->setTabBarPosition( CHexManagerWidget::BelowEditor );
    break;

    case CHexManagerWidget::AboveEditor:
    default:
      mDocumentTabs->setItemChecked( mDocumentTabsHideMB, false );
      mDocumentTabs->setItemChecked( mDocumentTabsAboveMB, true );
      mDocumentTabs->setItemChecked( mDocumentTabsBelowMB, false );
      mManager->setTabBarPosition( CHexManagerWidget::AboveEditor );
    break;
  }
}



void KHexEdit::statusBarPressed( int id )
{
  if( id == 0 )
  {
    SDisplayInputMode &input = editor()->inputMode();
    input.readOnly = input.readOnly == true ? false : true;
    textView()->setInputMode( input );
  }
  else if( id == 2 )
  {
    SDisplayLayout &layout = editor()->layout();
    if( layout.offsetMode == SDisplayLayout::hexadecimal )
    {
      layout.offsetMode = SDisplayLayout::decimal;
    }
    else
    {	
      layout.offsetMode = SDisplayLayout::hexadecimal;
    }
    textView()->setLayout( layout );
  }
}



void KHexEdit::open( QStringList &fileList, QStringList &offsetList )
{
  //
  // We load in reverse to allow the first document in 
  // the list to become the active one. 
  //
  
  uint offset;
  for( uint i = fileList.count(); i>0; i-- )
  {
    if( i <= offsetList.count() )
    {
      offset = (*offsetList.at(i-1)).toUInt(0,16);
    }
    else
    {
      offset = 0;
    }
    editor()->open( *fileList.at(i-1), true, offset );
  }
}

void KHexEdit::addRecentFile( const QString &fileName )
{
  if( fileName.isEmpty() == true )
  {
    return;
  }
  
  if( fileName.contains( i18n( "Untitled" ), false ) )
  {
    return;
  }

  uint recentCount = mRecent->count();
  for( uint id=0; id < recentCount; id++ )
  {
    QString text = mRecent->text(id);
    if( text.isNull() == true )
    {
      continue;
    }
 
    if( fileName == text )
    {
      mRecent->removeItem(id);
    }
  }
  
  while( mRecent->count() >= 5 )
  {
    mRecent->removeItemAt( 4 );
  }

  mRecent->insertItem( fileName, 0, 0 );
  for( uint i = 0; i < 5; i++ ) 
  {
    mRecent->setId( i, i );
  }
}


void KHexEdit::removeRecentFile( const QString &fileName )
{
  if( fileName.isEmpty() == true )
  {
    return;
  }
  
  if( fileName.contains( i18n( "Untitled" ), false ) )
  {
    return;
  }

  uint recentCount = mRecent->count();
  uint i;

  for( i=0; i < recentCount; i++ )
  {
    const char *text = mRecent->text(i);
    if( text == 0 )
    {
      continue;
    }
 
    if( fileName == text )
    { 
      mRecent->removeItem(i);
    }
  }
  for( i = 0; i < 5; i++ ) 
  {
    mRecent->setId( i, i );
  }
}


void KHexEdit::renameRecentFile(const QString &curName, const QString &newName)
{
  if( curName.contains( i18n( "Untitled" ), false ) )
  {
    addRecentFile( newName );
  }
  else
  {
    for( uint i=0; i < mRecent->count(); i++ )
    {
      const char *text = mRecent->text(i);
      if( text == 0 ) { continue; }
 
      if( curName == text )
      { 
	mRecent->changeItem( newName, i );
	break;
      }
    }
  }
}




void KHexEdit::recentFile( int id )
{
  QString url = mRecent->text(id);
  if( url.isNull() == false )
  {
    editor()->open( url, false, 0 );
  }
}


KHexEdit *KHexEdit::newWindow( void )
{
  KHexEdit *app = new KHexEdit;
  if( app == 0 || app->isValid() == 0 )
  {
    QString msg = i18n( "Can not create new window.\n" );
    msg += hexError( Err_NoMemory );
    KMessageBox::error( topLevelWidget(), msg, i18n("New window" ) );
    return(0);
  }
  app->show();
  return(app);
}



void KHexEdit::pasteNewWindow( void )
{
  KHexEdit *app = newWindow();
  if( app != 0 )
  {
    app->editor()->pasteNewFile();
  }
}



void KHexEdit::closeWindow( void )
{
  close();
}


void KHexEdit::closeProgram( void )
{
  if( mWindowList.count() > 1 )
  {
    for( KHexEdit *w = mWindowList.first(); w != 0; w = mWindowList.next() )
    {
      if( w->editor()->modified() == true )
      {
	QString msg = i18n(""
	  "There are windows with unsaved modified documents.\n"
	  "If you quit now, these modifications will be lost.\n\n"
	  "Proceed?");
	int reply = KMessageBox::warningYesNo( topLevelWidget(), msg );
	if( reply == KMessageBox::Yes )
	{
	  break;
	}
	else
	{
	  return;
	}
      }
    }

    //
    // Save the order in the Recent queue to a list. This list will be written 
    // to disk. 
    //
    makeExitFileList();

    //
    // Ths will write current configuration to disk.
    //
    queryExit();

    //
    // Now we need to destroy the manager widget. The editor widget 
    // (which is a child of the manager widget) will delete some
    // dialogs when destroyed. These dialogs have no parent (parent=0)
    // and sometimes their destructors are executed before the editor
    // widget are destroyed (unless we do as below). That's bad.
    //
    for( KHexEdit *w = mWindowList.first(); w != 0; w = mWindowList.next() )
    {
      delete w->mManager; w->mManager = 0;
    }
    kapp->quit();

  }
  else
  {
    closeWindow();
  }
 
}
    

bool KHexEdit::queryClose( void )
{
  //
  // Save the order in the Recent queue to a list. This list will be written 
  // to disk. 
  //
  makeExitFileList();

  bool success = editor()->closeAll();
  if( success == true )
  {
    mWindowList.removeRef( this );
  }

  return( success );
}


//
// We will always return true here. 
//
bool KHexEdit::queryExit( void )
{
  writeConfiguration();
  return( true );
}


void KHexEdit::makeExitFileList( void )
{
  //
  // Note: I append five names, even if they are empty because the recent 
  // menu contains fewer. This is important because otherwise we may end up 
  // having duplicate entries in the config. file.
  //

  mExitFileList.clear();
  mExitOffsetList.clear();
 
  if( editor()->discardRecentFiles() == true )
  {
    for( uint i=0; i < 5; i++ )
    {
      mExitFileList.append(QString(""));
      mExitOffsetList.append( QString("0") );
    }
  }
  else
  {
    uint offset;
    for( uint i=0; i < 5; i++ )
    {
      mExitFileList.append(mRecent->text(i));
      if( editor()->isOpen( mRecent->text(i), offset ) == true )
      {
	mExitOffsetList.append( QString("%1").arg(offset,0,16) );
      }
      else
      {
	mExitOffsetList.append( QString("%1").arg(0,0,16) );
      }
    }
  }
}


void KHexEdit::readConfiguration( void )
{
  KConfig &config = *kapp->config();
  readConfiguration( config );
}


void KHexEdit::writeConfiguration( void )
{
  KConfig &config = *kapp->config();
  writeConfiguration( config );
}


void KHexEdit::writeConfiguration( KConfig &config )
{
  config.setGroup("General Options" );
  config.writeEntry("Width", width() );
  config.writeEntry("Height", height() );
  config.writeEntry("ShowToolBar", mShowToolBar );
  config.writeEntry("ShowStatusBar", mShowStatusBar );
  config.writeEntry("ConversionPosition", (int)mManager->conversionPosition());
  config.writeEntry("TabBarPosition", (int)mManager->tabBarPosition());

  for( uint i=0; i < mExitFileList.count(); i++ )
  {
    config.writeEntry( QString("Recent%1").arg(i+1), *mExitFileList.at(i) );
  }
  config.writeEntry( QString("Offset"), mExitOffsetList );

  converter()->writeConfiguration( config );
  editor()->writeConfiguration( config );

  config.sync();
}


void KHexEdit::readConfiguration( KConfig &config )
{
  config.setGroup("General Options");
  mShowToolBar    = config.readBoolEntry( "ShowToolBar", true );
  mShowStatusBar  = config.readBoolEntry( "ShowStatusBar", true );
  int position    = config.readNumEntry( "ConversionPosition", 
					 CHexManagerWidget::InEditor );
  setConversionVisibility( (CHexManagerWidget::EConversionPosition)position );

  position = config.readNumEntry( "TabBarPosition", CHexManagerWidget::Hide );
  setDocumentTabBarVisibility( (CHexManagerWidget::ETabPosition)position );

  for( uint i = 0; i < 5; i++ ) 
  {
    QString fileName = config.readEntry( QString("Recent%1").arg(i+1) );
    if( fileName.isEmpty() == false )
    { 
      int pos = mRecent->count();
      mRecent->insertItem( fileName, pos, pos );
    }
  }
 
  mRecentOffsetList = config.readListEntry(QString("Offset") );

  converter()->readConfiguration( config );
  editor()->readConfiguration( config );
}



void KHexEdit::setStartupGeometry( void )
{
  KConfig &config = *kapp->config();
  config.setGroup("General Options");
  int w = config.readNumEntry( "Width", -1 );
  int h = config.readNumEntry( "Height", -1 );
  if( w < 10 || h < 10 )
  {
    setGeometry( 0, 0, editor()->defaultTextWidth(), 500 );
  }
  else
  {
    setGeometry( 0, 0, w, h );
  }
}



void KHexEdit::operationChanged( bool state )
{
  toolBar()->setItemEnabled( toolbar_cancelOperation, state );
  mFile->setItemEnabled( mFileCancelOperationMB, state );
}







void KHexEdit::cursorChanged( SCursorState &state )
{
  QString bit( i18n("Bit:") );
  QString offset( i18n("Offset:") );

  if( state.valid == false )
  {
    statusBar()->changeItem( bit, 1 );
    statusBar()->changeItem( offset, 2 );
  }
  else
  {
    SDisplayLayout &layout = editor()->layout();
 
    bit += QString().sprintf(" %u", state.cell );
    statusBar()->changeItem( bit, 1 );

    if( layout.offsetMode == SDisplayLayout::hexadecimal )
    {
      if( layout.offsetUpperCase == false )
      {
	offset += QString().sprintf( " %04x:%04x", state.offset>>16,
				     state.offset&0x0000FFFF );
      }
      else
      {
	offset += QString().sprintf( " %04X:%04X", state.offset>>16,
				     state.offset&0x0000FFFF );
      }
    }
    else
    {
      offset += QString().sprintf( " %010u", state.offset );
    }

    statusBar()->changeItem( offset, 2 );
  }
  
  setUndoState( state.undoState );
  setSelectionState( state.selectionSize == 0 ? false : true );
}




void KHexEdit::fileState( SFileState &state )
{
  if( state.valid == true )
  {
    statusBar()->changeItem( i18n("Size: %1").arg( state.size ), 3 );
    statusBar()->changeItem( state.modified == true ? "!" : "", 5 );
  }
  else
  {
    statusBar()->changeItem( "", 3 );
    statusBar()->changeItem( "", 5 );
  }
}



void KHexEdit::layoutChanged( const SDisplayLayout &layout )
{
  if( layout.primaryMode == SDisplayLayout::hexadecimal ) 
  {
    mView->setItemChecked( mViewHexMB, true );
    mEditorPopup->setItemChecked( mPopHexMB, true );
  }
  else
  {
    mView->setItemChecked( mViewHexMB, false );
    mEditorPopup->setItemChecked( mPopHexMB, false );
  }

  if( layout.primaryMode == SDisplayLayout::octal )
  { 
    mView->setItemChecked( mViewOctalMB, true );
    mEditorPopup->setItemChecked( mPopOctalMB, true );
  }
  else
  {
    mView->setItemChecked( mViewOctalMB, false );
    mEditorPopup->setItemChecked( mPopOctalMB, false );
  }

  if( layout.primaryMode == SDisplayLayout::binary )
  { 
    mView->setItemChecked( mViewBinaryMB, true );
    mEditorPopup->setItemChecked( mPopBinaryMB, true );
  }
  else
  {
    mView->setItemChecked( mViewBinaryMB, false );
    mEditorPopup->setItemChecked( mPopBinaryMB, false );
  }
  
  if( layout.primaryMode == SDisplayLayout::textOnly )
  { 
    mView->setItemChecked( mViewTextOnlyMB, true );
    mView->setItemEnabled( mViewTextAreaMB, false );
    mEditorPopup->setItemChecked( mPopTextOnlyMB, true );
  }
  else
  {
    mView->setItemChecked( mViewTextOnlyMB, false );
    mView->setItemEnabled( mViewTextAreaMB, true );
    mEditorPopup->setItemChecked( mPopTextOnlyMB, false );
  }

  mView->setItemChecked( mViewOffsetColumnMB, layout.offsetVisible );
  //mView->setItemEnabled( mViewBigCaseOffsetMB, layout.offsetVisible );

  if( layout.offsetMode == SDisplayLayout::hexadecimal )
  {
    mView->setItemChecked( mViewOffsetAsDecimalMB, false );
  }
  else
  {
    mView->setItemChecked( mViewOffsetAsDecimalMB, true );
  }

  if( layout.secondaryMode == SDisplayLayout::hide )
  {
    mView->setItemChecked( mViewTextAreaMB, false );
  }
  else
  {
    mView->setItemChecked( mViewTextAreaMB, true );
  }

  mView->setItemChecked( mViewBigCaseOffsetMB, layout.offsetUpperCase );
  mView->setItemChecked( mViewBigCaseDataMB, layout.primaryUpperCase );
}


void KHexEdit::inputModeChanged( const SDisplayInputMode &mode )
{
  mFilePermission->setItemChecked( mPermissionReadOnlyMB, mode.readOnly );
  mFilePermission->setItemChecked( mPermissionResizeMB, mode.allowResize );
  mEditorPopup->setItemChecked( mPopReadOnlyMB, mode.readOnly );
  mEditorPopup->setItemChecked( mPopResizeMB, mode.allowResize );
  statusBar()->changeItem( mode.readOnly == true ? i18n("R") : i18n("RW"), 0 );
 
  KIconLoader &loader = *KGlobal::iconLoader();
  QPixmap pixmap;
  if( mode.readOnly == false )
  {
    pixmap = loader.loadIcon( "hexwrite" ); // Custom pixmap
  }
  else
  {
    pixmap = loader.loadIcon( "lock");
  }
  KToolBarButton *tb =  toolBar(0)->getButton(toolbar_rw);
  tb->setPixmap( pixmap );
}



void KHexEdit::bookmarkChanged( QList<SCursorOffset> &list )
{
  if( mBookmark->count() == 4 && list.count() == 0 )
  {
    return;
  }

  mBookmark->clear();
  mBookmarkPopup->clear();

  uint entry = 4;
  
  mBookmark->insertItem( i18n("&Add Bookmark"), 0, 0 );
  mBookmark->setAccel( CTRL+Key_B, 0 );
  mBookmark->insertItem( i18n("&Replace Bookmark"), 1, 1 );
  mBookmark->setAccel( CTRL+Key_E, 1 );
  mBookmark->insertItem( i18n("R&emove Bookmark"), 2, 2 );
  mBookmark->setAccel( CTRL+Key_U, 2 );
  mBookmark->insertItem( i18n("Re&move All"), 3, 3 );

  if( list.count() == 0 )
  {
    return;
  }
  mBookmark->insertSeparator( entry++ );

  QString text1, text2, offset; 
  for( uint i=0; i < list.count(); i++ )
  {
    const SCursorOffset *p = list.at(i);
    if( p == 0 ) { continue; }

    offset.sprintf("%04X:%04X", p->offset>>16, p->offset&0x0000FFFF );
    text1 = i18n("Offset: %1").arg(offset);

    mBookmark->insertItem( text1, i+entry, i+entry );  
    int key = acceleratorNumKey( i );
    if( key > 0 )
    {
      mBookmark->setAccel( ALT+key, i+entry );
    }

    text2.sprintf( "[%u]", i+1 );
    text2 += " " + text1;

    mBookmarkPopup->insertItem( text2, i, i );
  }

}


void KHexEdit::fileActive( const QString &url )
{
  setCaption( url );
  mUrlLabel->setUrl( url );

  if( url.isNull() == false )
  {
    addDocument( url );
    addRecentFile( url );
  }
}


void KHexEdit::fileRename( const QString &curName, const QString &newName )
{
  if( newName.isNull() == true )
  {
    return;
  }

  renameDocument( curName, newName );
  renameRecentFile( curName, newName );
  setCaption( newName );
}



void KHexEdit::fileClosed( const QString &url )
{
  removeDocument( url );
}



void KHexEdit::editMode( CHexBuffer::EEditMode editMode )
{
  statusBar()->changeItem( editMode == CHexBuffer::edit_overwrite ? 
			   i18n("OVR") : i18n("INS"), 4 );
}


void KHexEdit::encodingChanged( const SEncodeState &state )
{    
  bool mode = state.mode == CConversion::cnvDefault ? true : false;
  mEncoding->setItemChecked( mEncodingDefaultMB, mode );
  mode = state.mode == CConversion::cnvEbcdic ? true : false;
  mEncoding->setItemChecked( mEncodingEbcdicMB, mode );
  mode = state.mode == CConversion::cnvUsAscii ? true : false;
  mEncoding->setItemChecked( mEncodingUsAsciiMB, mode );
  statusBar()->changeItem( QString(state.name), 6 );
}



void KHexEdit::textWidth( uint /*width*/ )
{
  setGeometry( x(), y(), editor()->defaultTextWidth(), height() );
}


void KHexEdit::viewMenuCB( int id )
{
  SDisplayLayout &layout = editor()->layout();
  SDisplayLine &line = editor()->line();

  switch( id )
  {
    case view_hexadecimal:
      layout.primaryMode   = SDisplayLayout::hexadecimal;
      layout.lineSize      = line.lineSize[ SDisplayLine::hexadecimal ];
      layout.columnSize    = line.columnSize[ SDisplayLine::hexadecimal ];
      textView()->setLayout( layout );
    break;

    case view_octal:
      layout.primaryMode   = SDisplayLayout::octal;
      layout.lineSize      = line.lineSize[ SDisplayLine::octal ];
      layout.columnSize    = line.columnSize[ SDisplayLine::octal ];
      textView()->setLayout( layout );
    break;

    case view_binary:
      layout.primaryMode   = SDisplayLayout::binary;
      layout.lineSize      = line.lineSize[ SDisplayLine::binary ];
      layout.columnSize    = line.columnSize[ SDisplayLine::binary ];
      textView()->setLayout( layout );
    break;

    case view_text:
      layout.primaryMode   = SDisplayLayout::textOnly;
      layout.lineSize      = line.lineSize[ SDisplayLine::textOnly ];
      layout.columnSize    = line.columnSize[ SDisplayLine::textOnly ];
      textView()->setLayout( layout );
    break;

    case view_offset_area:
      layout.offsetVisible = layout.offsetVisible == true ? false : true;
      textView()->setLayout( layout );
    break;

    case view_text_area:
      if( layout.secondaryMode == SDisplayLayout::hide )
      {
	layout.secondaryMode = SDisplayLayout::textOnly;
      }
      else
      {
	layout.secondaryMode = SDisplayLayout::hide;
      }
      textView()->setLayout( layout );
    break;

    case view_offset_as_decimal:
      if( layout.offsetMode == SDisplayLayout::hexadecimal )
      {
	layout.offsetMode = SDisplayLayout::decimal;
      }
      else
      {	
	layout.offsetMode = SDisplayLayout::hexadecimal;
      }
      textView()->setLayout( layout );
    break;

    case view_case_data:
      if( layout.primaryUpperCase == true )
      {	
	layout.primaryUpperCase = false;
      }
      else
      {
	layout.primaryUpperCase = true;
      }
      textView()->setLayout( layout );
    break;

    case view_case_offset:
      if( layout.offsetUpperCase == true )
      {
	layout.offsetUpperCase = false;
      }
      else
      {
	layout.offsetUpperCase = true;
      }
      textView()->setLayout( layout );
    break;

    case view_lock_width:
      layout.lockLine = layout.lockLine == True ? False : True;
      textView()->setLayout( layout );
    break;

    case permission_readonly:
      editor()->toggleWriteProtection();
    break;

    case permission_allowResize:
    {
      SDisplayInputMode &input = editor()->inputMode();
      input.allowResize = input.allowResize == true ? false : true;
      textView()->setInputMode( input );
    }
    break;
  }
}


void KHexEdit::encodingMenuCB( int id )
{
  switch( id )
  {
    case encoding_default:
      editor()->encode( CConversion::cnvDefault );
    break;

    case encoding_us_ascii:
      editor()->encode( CConversion::cnvUsAscii );
    break;

    case encoding_ebcdic:
      editor()->encode( CConversion::cnvEbcdic );
    break;
  }
}


void KHexEdit::bookmarkMenuCB( int id )
{
  if( id == 0 )
  {
    editor()->addBookmark();
  }
  else if( id == 1 )
  {
    editor()->replaceBookmark();
  }
  else if( id == 2 )
  {
    editor()->removeBookmark( false );
  }
  else if( id == 3 )
  {
    editor()->removeBookmark( true );
  }
  else if( id >= 5 )
  {
    editor()->gotoBookmark( id - 5 );
  }
}


void KHexEdit::bookmarkPopupCB( int id )
{
  editor()->gotoBookmark( id );
}


void KHexEdit::optionMenuCB( int id )
{
  switch( id )
  {
    case option_toolbar:
      mShowToolBar = mShowToolBar == true ? false : true;
      mOptions->setItemChecked( mOptionToolBarMB, mShowToolBar );
      if( mShowToolBar == true )
      {
	enableToolBar(KToolBar::Show);
      }
      else
      {
	enableToolBar(KToolBar::Hide);
      }
    break;

    case option_statusbar:
      mShowStatusBar = mShowStatusBar == true ? false : true;
      mOptions->setItemChecked( mOptionStatusBarMB, mShowStatusBar );
      if( mShowStatusBar == true )
      {
	enableStatusBar(KStatusBar::Show);
      }
      else
      {
	enableStatusBar(KStatusBar::Hide);
      }
    break;

    case conversion_hide:
      setConversionVisibility( CHexManagerWidget::Hide );
    break;

    case conversion_window:
      setConversionVisibility( CHexManagerWidget::InWindow );
    break;

    case conversion_editor:
      setConversionVisibility( CHexManagerWidget::InEditor );
    break;

    case documenttabs_hide:
      setDocumentTabBarVisibility( CHexManagerWidget::HideTab );
    break;

    case documenttabs_above:
      setDocumentTabBarVisibility( CHexManagerWidget::AboveEditor );
    break;

    case documenttabs_below:
      setDocumentTabBarVisibility( CHexManagerWidget::BelowEditor );
    break;

    case option_edit:
      editor()->options();
    break;

    case option_favorites:
      editor()->favorites();
    break;
  }
}





void KHexEdit::setUndoState( uint undoState ) 
{
  if( mUndoState != undoState )
  {
    mUndoState = undoState;
    mEdit->setItemEnabled( mEditUndoMB, mUndoState & UNDO_OK );
    mEdit->setItemEnabled( mEditRedoMB, mUndoState & REDO_OK );
    toolBar()->setItemEnabled( toolbar_Undo, mUndoState & UNDO_OK );
    toolBar()->setItemEnabled( toolbar_Redo, mUndoState & REDO_OK );
  }
}


void KHexEdit::setSelectionState( bool selectionValid )
{
  if( mSelectionValid != selectionValid )
  {
    mSelectionValid = selectionValid;
    mEdit->setItemEnabled( mEditCopyMB, mSelectionValid );
    mEdit->setItemEnabled( mEditCutMB, mSelectionValid );
    mEditSpecial->setItemEnabled( mEditSpecialCopyText1MB, mSelectionValid );
    mEditSpecial->setItemEnabled( mEditSpecialCopyText2MB, mSelectionValid );

    toolBar()->setItemEnabled( toolbar_Copy, mSelectionValid );
    toolBar()->setItemEnabled( toolbar_Cut, mSelectionValid );
  }
}



void KHexEdit::documentMenuCB( int index )
{
  if( (uint)index < mDocumentList.count() )
  {
    editor()->open( mDocumentList[index], false, 0 );
  }
}



void KHexEdit::addDocument( const QString &fileName )
{
  uint documentCount = mDocumentList.count();
  for( uint i=0; i < documentCount; i++ )
  {
    if( fileName == mDocumentList[i] )
    {
      return;
    }
  }

  mDocumentList.append( fileName );
  mDocuments->insertItem( fileName, documentCount, documentCount );
  int key = acceleratorNumKey( documentCount );
  if( key > 0 )
  {
    mDocuments->setAccel( CTRL+key, documentCount );
  }
}



void KHexEdit::removeDocument( const QString &fileName )
{
  QStringList::Iterator it;
  for( it = mDocumentList.begin(); it != mDocumentList.end(); it++ )
  {
    if( *it == fileName )
    {
      mDocuments->removeItemAt( mDocumentList.findIndex(*it) );
      mDocumentList.remove( it );
 
      for( uint i = 0; i < mDocumentList.count(); i++ ) 
      {
	mDocuments->setId( i, i );
	int key = acceleratorNumKey( i );
	if( key > 0 )
	{
	  mDocuments->setAccel( CTRL+key, i );
	}
      }
      return;
    }
  }
}



void KHexEdit::renameDocument( const QString &curName, const QString &newName )
{
  QStringList::Iterator it;
  for( it = mDocumentList.begin(); it != mDocumentList.end(); it++ )
  {
    if( *it == curName )
    {
      mDocuments->changeItem( newName, mDocumentList.findIndex(*it) );
      mDocumentList.insert( it, newName );
      mDocumentList.remove( it );
      return;
    }
  }
}


void KHexEdit::conversionClosed( void )
{
  setConversionVisibility( CHexManagerWidget::Hide );
}


void KHexEdit::showAboutApplication( void )
{
  if( mAboutKHexEdit == 0 )
  {
    mAboutKHexEdit = new CAboutKHexEdit( topLevelWidget(), 0, false );
    if( mAboutKHexEdit == 0 ) { return; }
  }

  if( mAboutKHexEdit->isVisible() == false )
  {
    mAboutKHexEdit->show();
  }
  else
  {
    mAboutKHexEdit->raise();
  }
}


bool KHexEdit::eventFilter( QObject *, QEvent *event )
{
  if( event->type() == QEvent::DragEnter )
  {
    QDragEnterEvent *e = (QDragEnterEvent*)event;
    if( QUriDrag::canDecode(e) )
    {
      e->accept();
      textView()->setDropHighlight( true );
    }
    return( true );
  }
  else if( event->type() == QEvent::Drop )
  {
    QMimeSource &m = *(QDropEvent*)event;
    textView()->setDropHighlight( false );
    
    QStringList list, offset;
    if( QUriDrag::decodeToUnicodeUris( &m, list ) == true )
    {
      open( list, offset );
      return( true );
    }
    return( true );
  }
  else if( event->type() == QEvent::DragLeave )
  {
    textView()->setDropHighlight( false );
    return( true );
  }

  return( false );
}
